@@ -149,14 +149,23 @@ public:
      */
     virtual void toBuffer(DataBlock& buf);
 
+    /**
+     * Add this element to a string
+     * @param buf Destination string
+     */
+    virtual void toString(String& buf);
+
     /**
      * Get the text associated with an IE type value
      * @param ieCode Numeric code of the IE
      * @return Pointer to the IE text or 0 if it doesn't exist
      */
-    static const char* ieText(u_int8_t ieCode);
+    static inline const char* ieText(u_int8_t ieCode)
+	{ return lookup(ieCode,s_ieData); }
+
 
 private:
+    static TokenDict s_ieData[];// Association between IE type and text
     Type m_type;		// Type of this IE
 };
 
@@ -201,6 +210,13 @@ public:
      */
     virtual void toBuffer(DataBlock& buf);
 
+    /**
+     * Add this element to a string
+     * @param buf Destination string
+     */
+    virtual void toString(String& buf)
+	{ buf << m_strData; }
+
 private:
     String m_strData;		// IE text data
 };
@@ -245,6 +261,12 @@ public:
      */
     virtual void toBuffer(DataBlock& buf);
 
+    /**
+     * Add this element to a string
+     * @param buf Destination string
+     */
+    virtual void toString(String& buf);
+
 private:
     u_int8_t m_length;		// IE data length
     u_int32_t m_numericData;	// IE numeric data
@@ -306,6 +328,12 @@ public:
      */
     static bool unpackIP(SocketAddr& addr, IAXInfoElementBinary* ie);
 
+    /**
+     * Add this element to a string
+     * @param buf Destination string
+     */
+    virtual void toString(String& buf);
+
 private:
     DataBlock m_data;		// IE binary data
 };
@@ -326,9 +354,10 @@ public:
     /**
      * Constructor. Construct the list from an IAXFullFrame object
      * @param frame Source object
+     * @param incoming True if it is an incoming frame
      */
-    inline IAXIEList(const IAXFullFrame* frame) : m_invalidIEList(false)
-	{ createFromFrame(frame); }
+    inline IAXIEList(const IAXFullFrame* frame, bool incoming = true) : m_invalidIEList(false)
+	{ createFromFrame(frame,incoming); }
 
     /**
      * Destructor
@@ -417,9 +446,10 @@ public:
      * Construct the list from an IAXFullFrame object.
      *  On exit m_invalidIEList will contain the opposite of the returned value
      * @param frame Source object
+     * @param incoming True if it is an incoming frame
      * @return False if the frame contains invalid IEs
      */
-    bool createFromFrame(const IAXFullFrame* frame);
+    bool createFromFrame(const IAXFullFrame* frame, bool incoming = true);
 
     /**
      * Construct a buffer from this list
@@ -427,6 +457,13 @@ public:
      */
     void toBuffer(DataBlock& buf);
 
+    /**
+     * Add this list to a string
+     * @param dest Destination string
+     * @param indent Optional indent for each element
+     */
+    void toString(String& dest, const char* indent = 0);
+
     /**
      * Get an IAXInfoElement from the list
      * @param type The desired type
@@ -478,6 +515,16 @@ public:
         MD5  = 2,
         RSA  = 4,
     };
+
+    /**
+     * Create a string list from authentication methods
+     * @param dest The destination
+     * @param formats The authentication methods
+     * @param sep The separator to use
+    */
+    static void authList(String& dest, u_int16_t auth, char sep);
+
+    static TokenDict s_texts[];
 };
 
 /**
@@ -514,19 +561,29 @@ public:
         H263   = (1 << 19),
     };
 
+    /**
+     * Create a string list from formats
+     * @param dest The destination
+     * @param formats The formats
+     * @param sep The separator to use
+    */
+    static void formatList(String& dest, u_int32_t formats, char sep = ',');
+
     /**
      * Get the text associated with an audio format
      * @param audio The desired format
      * @return A pointer to the text associated with the format or 0 if the format doesn't exist
     */
-    static const char* audioText(u_int32_t audio);
+    static inline const char* audioText(u_int32_t audio)
+	{ return lookup(audio,audioData); }
 
     /**
      * Get the text associated with a video format
      * @param video The desired format
      * @return A pointer to the text associated with the format or 0 if the format doesn't exist
     */
-    static const char* videoText(u_int32_t video);
+    static inline const char* videoText(u_int32_t video)
+	{ return lookup(video,videoData); }
 
     /**
      * Keep the texts associated with the audio formats
@@ -588,6 +645,17 @@ public:
         FwDownl   = 0x24,
         FwData    = 0x25,
     };
+
+    /**
+     * Get the string associated with the given IAX control type
+     * @param type The requested type
+     * @return The text if type is valid or 0
+     */
+    static inline const char* typeText(int type)
+	{ return lookup(type,s_types,0); }
+
+private:
+    static TokenDict s_types[]; // Keep the association between IAX control codes and their name
 };
 
 /**
@@ -695,6 +763,14 @@ public:
      */
     static u_int32_t unpackSubclass(u_int8_t value);
 
+    /**
+     * Get the string associated with the given IAX frame type
+     * @param type The requested type
+     * @return The text if type is valid or 0
+     */
+    static inline const char* typeText(int type)
+	{ return lookup(type,s_types,0); }
+
 protected:
     /**
      * Contains the frame's IE list for an incoming frame or the whole frame for an outgoing one
@@ -707,6 +783,7 @@ protected:
     bool m_retrans;
 
 private:
+    static TokenDict s_types[]; // Keep the association between IAX frame types and their names
     Type m_type;		// Frame type
     u_int16_t m_sCallNo;	// Source call number
     u_int32_t m_tStamp;		// Frame timestamp
@@ -814,7 +891,26 @@ public:
      */
     virtual const IAXFullFrame* fullFrame() const;
 
+    /**
+     * Fill a string with this frame
+     * @param dest The string to fill
+     * @param local The local address
+     * @param remote The remote address
+     * @param incoming True if it is an incoming frame
+     */
+    void toString(String& dest, const SocketAddr& local, const SocketAddr& remote,
+	bool incoming) const;
+
+    /**
+     * Get the string associated with the given IAX control type
+     * @param type The requested control type
+     * @return The text if type is valid or 0
+     */
+    static inline const char* controlTypeText(int type)
+	{ return lookup(type,s_controlTypes,0); }
+
 private:
+    static TokenDict s_controlTypes[]; // Keep the association between control types and their names
     u_int16_t m_dCallNo;	// Destination call number
     unsigned char m_oSeqNo;	// Out sequence number
     unsigned char m_iSeqNo;	// In sequence number
@@ -1123,13 +1219,6 @@ public:
     inline const String& username()
 	{ return m_username; }
 
-    /**
-     * Retrive the password
-     * @return A reference to the password
-     */
-    inline const String& password()
-	{ return m_password; }
-
     /**
      * Retrive the calling number
      * @return A reference to the calling number
@@ -1299,17 +1388,17 @@ public:
     /**
      * Send an AUTHREQ/REGAUTH frame to remote peer
      * This method is thread safe
-     * @param pwd Required password
      * @return False if the current transaction state is not NewRemoteInvite
      */
-    bool sendAuth(const String& pwd);
+    bool sendAuth();
 
     /**
      * Send an AUTHREP/REGREQ/REGREL frame to remote peer as a response to AUTHREQ/REGREQ/REGREL
      * This method is thread safe
+     * @param response Response to send
      * @return False if the current transaction state is not NewLocalInvite_AuthRecv
      */
-    bool sendAuthReply();
+    bool sendAuthReply(const String& response);
 
     /**
      * Send a DTMF frame to remote peer
@@ -1371,6 +1460,11 @@ public:
      */
     static String s_iax_modInvalidAuth;
 
+    /**
+     * Standard message sent if a received frame doesn't have an username information element
+     */
+    static String s_iax_modNoUsername;
+
 protected:
     /**
      * Constructor: constructs an incoming transaction from a received full frame with an IAX
@@ -1703,11 +1797,7 @@ protected:
      * This method is thread safe
      * @param event The event notifying termination
      */
-    inline void eventTerminated(IAXEvent* event) {
-	Lock lock(this);
-	if (event == m_currentEvent)
-	    m_currentEvent = 0;
-    }
+    void eventTerminated(IAXEvent* event);
 
     /**
      * Set the current event
@@ -1758,7 +1848,6 @@ private:
     // Data
     IAXAuthMethod::Type m_authmethod;		// Authentication method to use
     String m_username;				// Username
-    String m_password;				//  Password
     String m_callingNo;				// Calling number
     String m_callingName;			// Calling name
     String m_calledNo;				// Called number
@@ -1792,7 +1881,7 @@ public:
         Timeout,		// Transaction timeout
 	NotImplemented,		// Feature not implemented
 	New,			// New remote transaction
-	AuthReq,		// Auth request. Internally processed
+	AuthReq,		// Auth request
 	AuthRep,		// Auth reply
 	Accept,			// Request accepted
 	Hangup,			// Remote hangup
@@ -1938,10 +2027,11 @@ public:
      * @param format Default media format
      * @param capab Media capabilities of this engine
      * @param trunkSendInterval Send trunk meta frame interval
+     * @param authRequired Automatically challenge all clients for authentication
      */
     IAXEngine(const char* iface, int port, u_int16_t transListCount, u_int16_t retransCount, u_int16_t retransInterval,
 	u_int16_t authTimeout, u_int16_t transTimeout, u_int16_t maxFullFrameDataLen,
-	u_int32_t format, u_int32_t capab, u_int32_t trunkSendInterval);
+	u_int32_t format, u_int32_t capab, u_int32_t trunkSendInterval, bool authRequired);
 
     /**
      * Destructor
@@ -1996,6 +2086,13 @@ public:
     inline u_int16_t retransInterval() const
         { return m_retransInterval; }
 
+    /**
+     * Check if a transaction should automatically request authentication
+     * @return True to automatically request authentication
+     */
+    inline bool authRequired() const
+        { return m_authRequired; }
+
     /**
      * Get the timeout (in seconds) of acknoledged auth frames sent
      * @return Auth timeout in seconds
@@ -2042,9 +2139,10 @@ public:
      * @param buf Data to write
      * @param len Data length
      * @param addr Socket to write to
+     * @param frame Optional frame to be printed if debug is DebugAll
      * @return True on success
      */
-    bool writeSocket(const void* buf, int len, const SocketAddr& addr);
+    bool writeSocket(const void* buf, int len, const SocketAddr& addr, IAXFullFrame* frame = 0);
 
     /**
      * Read events
@@ -2186,13 +2284,15 @@ private:
     bool m_lUsedCallNo[IAX2_MAX_CALLNO + 1];	// Used local call numnmbers flags
     int m_lastGetEvIndex;			// getEvent: keep last array entry
     // Parameters
+    bool m_authRequired;			// Automatically request authentication
     int m_maxFullFrameDataLen;			// Max full frame data (IE list) length
     u_int16_t m_startLocalCallNo;		// Start index of local call number allocation
     u_int16_t m_transListCount;			// m_transList count
     u_int16_t m_retransCount;			// Retransmission counter for each transaction belonging to this engine
     u_int16_t m_retransInterval;		// Retransmission interval default value in miliseconds
     u_int16_t m_authTimeout;			// Timeout (in seconds) of acknoledged auth frames sent
-    u_int32_t m_transTimeout;			// Timeout (in seconds) on remote request of transactions belonging to this engine
+    u_int32_t m_transTimeout;			// Timeout (in seconds) on remote request of transactions
+    						//  belonging to this engine
     // Media
     u_int32_t m_format;				// The default media format
     u_int32_t m_capability;			// The media capability
