@@ -257,6 +257,7 @@ void RTPReceiver::timerTick(const Time& when)
 {
 }
 
+
 RTPSender::RTPSender(RTPSession* session, bool randomTs)
     : RTPBaseIO(session), m_evTime(0), m_tsLast(0)
 {
@@ -457,7 +458,7 @@ RTPTransport* RTPSession::createTransport()
     return trans;
 }
 
-bool RTPSession::initGroup()
+bool RTPSession::initGroup(int msec, Thread::Priority prio)
 {
     if (m_group)
 	return true;
@@ -465,7 +466,7 @@ bool RTPSession::initGroup()
     if (m_transport)
 	group(m_transport->group());
     if (!m_group)
-	group(new RTPGroup());
+	group(new RTPGroup(msec,prio));
     if (!m_group)
 	return false;
     if (m_transport)
