@@ -27,6 +27,8 @@
 
 using namespace TelEngine;
 
+static unsigned long s_sleep = 5;
+
 class RTPDelayedData : public DataBlock
 {
 public:
@@ -48,10 +50,15 @@ private:
 };
 
 
-RTPGroup::RTPGroup(Priority prio)
+RTPGroup::RTPGroup(int msec, Priority prio)
     : Mutex(true), Thread("RTP Group",prio), m_listChanged(false)
 {
     DDebug(DebugInfo,"RTPGroup::RTPGroup() [%p]",this);
+    if (msec < 1)
+	msec = 1;
+    if (msec > 50)
+	msec = 50;
+    m_sleep = msec;
 }
 
 RTPGroup::~RTPGroup()
@@ -83,6 +90,9 @@ void RTPGroup::run()
     DDebug(DebugInfo,"RTPGroup::run() [%p]",this);
     bool ok = true;
     while (ok) {
+	unsigned long msec = m_sleep;
+	if (msec < s_sleep)
+	    msec = s_sleep;
 	lock();
 	Time t;
 	ObjList* l = &m_processors;
@@ -100,7 +110,7 @@ void RTPGroup::run()
 	    }
 	}
 	unlock();
-	Thread::msleep(1,true);
+	Thread::msleep(msec,true);
     }
     DDebug(DebugInfo,"RTPGroup::run() ran out of processors [%p]",this);
 }
@@ -124,6 +134,15 @@ void RTPGroup::part(RTPProcessor* proc)
     unlock();
 }
 
+void RTPGroup::setMinSleep(int msec)
+{
+    if (msec < 1)
+	msec = 1;
+    if (msec > 20)
+	msec = 20;
+    s_sleep = msec;
+}
+
 
 RTPProcessor::RTPProcessor()
     : m_group(0)
@@ -180,6 +199,9 @@ void RTPTransport::timerTick(const Time& when)
 	SocketAddr addr;
 	int len;
 	while ((len = m_rtpSock.recvFrom(buf,sizeof(buf),addr)) >= 12) {
+	    if (((unsigned char)buf[0] & 0xc0) != 0x80)
+		continue;
+	    // looks like it's RTP, at least by version
 	    if (m_autoRemote && (addr != m_remoteAddr)) {
 		Debug(DebugInfo,"Auto changing RTP address from %s:%d to %s:%d",
 		    m_remoteAddr.host().c_str(),m_remoteAddr.port(),
@@ -194,6 +216,7 @@ void RTPTransport::timerTick(const Time& when)
 		    m_monitor->rtpData(buf,len);
 	    }
 	}
+	m_rtpSock.timerTick(when);
     }
     if (m_rtcpSock.valid()) {
 	char buf[BUF_SIZE];
@@ -205,6 +228,7 @@ void RTPTransport::timerTick(const Time& when)
 	    if (m_monitor)
 		m_monitor->rtcpData(buf,len);
 	}
+	m_rtcpSock.timerTick(when);
     }
 }
 
@@ -308,6 +332,19 @@ bool RTPTransport::remoteAddr(SocketAddr& addr, bool sniff)
     return false;
 }
 
+bool RTPTransport::drillHole()
+{
+    if (m_rtpSock.valid() && m_remoteAddr.valid()) {
+	static const char buf[4] = { 0, 0, 0, 0 };
+	if (m_rtpSock.sendTo(buf,sizeof(buf),m_remoteAddr) == sizeof(buf)) {
+	    if (m_rtcpSock.valid() && m_remoteRTCP.valid())
+		m_rtcpSock.sendTo(buf,sizeof(buf),m_remoteRTCP);
+	    return true;
+	}
+    }
+    return false;
+}
+
 
 RTPDejitter::RTPDejitter(RTPReceiver* receiver, unsigned int mindelay, unsigned int maxdelay)
     : m_receiver(receiver), m_mindelay(mindelay), m_maxdelay(maxdelay),
