@@ -126,9 +126,10 @@ class YRTP_API RTPGroup : public GenObject, public Mutex, public Thread
 public:
     /**
      * Constructor
+     * @param msec Minimum time to sleep in loop in milliseconds
      * @param prio Thread priority to run this group
      */
-    RTPGroup(Priority prio = Normal);
+    RTPGroup(int msec = 0, Priority prio = Normal);
 
     /**
      * Group destructor, removes itself from all remaining processors
@@ -145,6 +146,12 @@ public:
      */
     virtual void run();
 
+    /**
+     * Set the system global minimum time to sleep in loop
+     * @param msec Minimum time to sleep in loop in milliseconds
+     */
+    static void setMinSleep(int msec);
+
 protected:
     /**
      * Add a RTP processor to this group
@@ -161,6 +168,7 @@ protected:
 private:
     ObjList m_processors;
     bool m_listChanged;
+    unsigned long m_sleep;
 };
 
 /**
@@ -239,6 +247,19 @@ public:
     inline bool setTOS(int tos)
 	{ return m_rtpSock.setTOS(tos); }
 
+    /**
+     * Get the RTP socket used by this transport
+     * @return Pointer to the RTP socket
+     */
+    inline Socket* rtpSock()
+	{ return &m_rtpSock; }
+
+    /**
+     * Drill a hole in a firewall or NAT for the RTP and RTCP sockets
+     * @return True if at least a packet was sent for the RTP socket
+     */
+    bool drillHole();
+
 protected:
     /**
      * Method called periodically to read data out of sockets
@@ -720,9 +741,11 @@ public:
 
     /**
      * Initialize the RTP session, attach a group if none is present
+     * @param msec Minimum time to sleep in group loop in milliseconds
+     * @param prio Thread priority to run the new group
      * @return True if initialized, false on some failure
      */
-    bool initGroup();
+    bool initGroup(int msec = 0, Thread::Priority prio = Thread::Normal);
 
     /**
      * Send one RTP payload packet
@@ -898,6 +921,20 @@ public:
     inline bool setTOS(int tos)
 	{ return m_transport && m_transport->setTOS(tos); }
 
+    /**
+     * Get the RTP socket used by this session
+     * @return Pointer to the RTP socket, NULL if no transport exists
+     */
+    inline Socket* rtpSock()
+	{ return m_transport ? m_transport->rtpSock() : 0; }
+
+    /**
+     * Drill a hole in a firewall or NAT for the RTP and RTCP sockets
+     * @return True if at least a packet was sent for the RTP socket
+     */
+    inline bool drillHole()
+	{ return m_transport && m_transport->drillHole(); }
+
 protected:
     /**
      * Method called periodically to push any asynchronous data or statistics
