@@ -73,7 +73,7 @@ CallEndpoint::CallEndpoint(const char* id)
 {
 }
 
-CallEndpoint::~CallEndpoint()
+void CallEndpoint::destroyed()
 {
 #ifdef DEBUG
     ObjList* l = m_data.skipNull();
@@ -465,19 +465,8 @@ Message* Channel::message(const char* name, const NamedList* original, const cha
     if (original) {
 	if (!params)
 	    params = original->getValue("copyparams");
-	if (!null(params)) {
-	    String tmp(params);
-	    ObjList* lst = tmp.split(',',false);
-	    for (ObjList* l = lst; l; l = l->next()) {
-		String* s = static_cast<String*>(l->get());
-		if (!s)
-		    continue;
-		s->trimBlanks();
-		if (*s)
-		    msg->copyParam(*original,*s);
-	    }
-	    delete lst;
-	}
+	if (!null(params))
+	    msg->copyParams(*original,params);
     }
     return msg;
 }
@@ -493,7 +482,7 @@ bool Channel::startRouter(Message* msg)
 	delete r;
     }
     else
-	delete msg;
+	TelEngine::destruct(msg);
     callRejected("failure","Internal server error");
     // dereference and die if the channel is dynamic
     if (m_driver && m_driver->varchan())
@@ -1087,10 +1076,10 @@ bool Driver::received(Message &msg, int id)
 	    dest = msg.getValue("id");
 	    break;
 	default:
-	    dest = msg.getValue("targetid");
-	    // if this channel is not the target try to match it as peer
+	    dest = msg.getValue("peerid");
+	    // if this channel is not the peer, try to match it as target
 	    if (!dest.startsWith(m_prefix))
-		dest = msg.getValue("peerid");
+		dest = msg.getValue("targetid");
 	    break;
     }
     XDebug(DebugAll,"id=%d prefix='%s' dest='%s'",id,m_prefix.c_str(),dest.c_str());
@@ -1146,8 +1135,10 @@ bool Driver::received(Message &msg, int id)
 	    msg = msg.getValue("message");
 	    msg.clearParam("message");
 	    msg.userData(chan);
+	    if (chan->msgMasquerade(msg))
+		return true;
 	    chan->complete(msg);
-	    return chan->msgMasquerade(msg);
+	    return false;
 	case Locate:
 	    msg.userData(chan);
 	    return true;
@@ -1203,22 +1194,6 @@ void Driver::genUpdate(Message& msg)
     msg.addParam("chans",String(m_chans.count()));
 }
 
-void Driver::msgStatus(Message& msg)
-{
-    String mod, par, det;
-    bool details = msg.getBoolValue("details",true);
-    lock();
-    statusModule(mod);
-    statusParams(par);
-    if (details)
-	statusDetail(det);
-    unlock();
-    msg.retValue() << mod << ";" << par;
-    if (details)
-	msg.retValue() << ";" << det;
-    msg.retValue() << "\r\n";
-}
-
 void Driver::statusModule(String& str)
 {
     Module::statusModule(str);
@@ -1412,7 +1387,7 @@ bool Router::route()
 
 void Router::cleanup()
 {
-    delete m_msg;
+    destruct(m_msg);
 }
 
 /* vi: set ts=8 sw=4 sts=4 noet: */
