@@ -389,6 +389,8 @@ static Configuration s_contacts;
 static Configuration s_providers;
 static Configuration s_history;
 static unsigned int s_eventLen = 0;
+static Mutex s_debugMutex;
+static String* s_debugLog = 0;
 
 // Parameters that are stored with account
 static const char* s_accParams[] = {
@@ -408,6 +410,17 @@ static const char* s_provParams[] = {
     0
 };
 
+static void dbg_client_func(const char *buf, int level)
+{
+    if (!buf)
+	return;
+    s_debugMutex.lock();
+    if (s_debugLog)
+	*s_debugLog += buf;
+    else
+	s_debugLog = new String(buf);
+    s_debugMutex.unlock();
+}
 
 Client::Client(const char *name)
     : Thread(name), m_initialized(false), m_line(0), m_oneThread(true),
@@ -432,14 +445,16 @@ void Client::run()
     loadWindows();
     Message msg("ui.event");
     msg.setParam("event","load");
-    Engine::dispatch(msg);
+    if (Engine::dispatch(msg))
+	Debug(DebugGoOn,"Message %s was unexpectedly handled!",msg.c_str());
     initWindows();
     initClient();
     updateFrom(0);
     setStatus(Engine::config().getValue("client","greeting","Yate " YATE_VERSION " - " YATE_RELEASE));
     m_initialized = true;
     msg.setParam("event","init");
-    Engine::dispatch(msg);
+    if (Engine::dispatch(msg))
+	Debug(DebugGoOn,"Message %s was unexpectedly handled!",msg.c_str());
     main();
     exitClient();
 }
@@ -499,7 +514,7 @@ void Client::initWindows()
 
 void Client::initClient()
 {
-    s_eventLen = Engine::config().getIntValue("client","eventlen",4096);
+    s_eventLen = Engine::config().getIntValue("client","eventlen",10240);
     if (s_eventLen > 65535)
 	s_eventLen = 65535;
     else if (s_eventLen && (s_eventLen < 1024))
@@ -1013,9 +1028,13 @@ bool Client::getSelect(const String& name, String& item, Window* wnd, Window* sk
 
 bool Client::addToLog(const String& text, Window* wnd)
 {
+    if (text.null())
+	return true;
     String tmp;
     if (getText("log_events",tmp,wnd)) {
-	tmp.append(text,"\n");
+	if (tmp && tmp.at(tmp.length()-1) != '\n')
+	    tmp += "\n";
+	tmp += text;
 	while (s_eventLen && (tmp.length() > s_eventLen)) {
 	    int pos = tmp.find('\n');
 	    if (pos < 0)
@@ -1084,7 +1103,11 @@ bool Client::action(Window* wnd, const String& name)
 	getText("account",account,wnd);
 	account.trimBlanks();
 	fixDashes(account);
-	return callStart(target,line,proto,account);
+	if (!callStart(target,line,proto,account))
+	    return false;
+	delOption("callto",target,wnd);
+	addOption("callto",target,true,target,wnd);
+	return true;
     }
     else if (name.startsWith("callto:"))
 	return callStart(name.substr(7));
@@ -1158,6 +1181,24 @@ bool Client::action(Window* wnd, const String& name)
 	    }
 	}
     }
+    else if (name.startsWith("command:") && name.at(8)) {
+	// generic engine commands
+	Message* m = new Message("engine.command");
+	m->addParam("line",name.c_str()+8);
+	Engine::enqueue(m);
+	return true;
+    }
+    else if (name.startsWith("debug:")) {
+	// module debugging control commands
+	int sep = name.find(':',6);
+	if (sep > 0) {
+	    Message* m = new Message("engine.debug");
+	    m->addParam("module",name.substr(6,sep-6));
+	    m->addParam("line",name.substr(sep+1));
+	    Engine::enqueue(m);
+	    return true;
+	}
+    }
     // accounts window actions
     else if (name == "acc_new") {
 	NamedList params("");
@@ -1512,6 +1553,25 @@ bool Client::toggle(Window* wnd, const String& name, bool active)
 	if (setShow(name.substr(8),active,wnd))
 	    return true;
     }
+    else if (name.startsWith("debug:")) {
+	// module debugging control commands
+	int sep = name.find(':',6);
+	if (sep > 0) {
+	    String line = name.substr(sep+1);
+	    int sep2 = line.find(':');
+	    if (sep2 > 0)
+		line = active ? line.substr(0,sep2) : line.substr(sep2+1);
+	    else if (!active)
+		return true;
+	    if (line) {
+		Message* m = new Message("engine.debug");
+		m->addParam("module",name.substr(6,sep-6));
+		m->addParam("line",line);
+		Engine::enqueue(m);
+	    }
+	    return true;
+	}
+    }
     // keep the toggle in sync in all windows
     setCheck(name,active,0,wnd);
     if (name == "autoanswer") {
@@ -1522,6 +1582,19 @@ bool Client::toggle(Window* wnd, const String& name, bool active)
 	m_multiLines = active;
 	return true;
     }
+    if (name == "log_events_debug") {
+	Debug(ClientDriver::self(),DebugNote,"Debug to window: %s",String::boolText(active));
+	setShow("log_events_control",active,wnd);
+	if (active) {
+	    ClientDriver::self()->debugEnabled(false);
+	    Debugger::setOutput(dbg_client_func);
+	}
+	else {
+	    Debugger::setOutput(0);
+	    ClientDriver::self()->debugEnabled(true);
+	}
+	return true;
+    }
 
     // unknown/unhandled - generate a message for them
     Message* m = new Message("ui.event");
@@ -1569,10 +1642,17 @@ bool Client::select(Window* wnd, const String& name, const String& item, const S
 	if (!sect)
 	    return false;
 	setSelect("acc_protocol",sect->getValue("protocol"));
+	String acc = item;
+	// make sure we offer an unique account name
+	for (unsigned int i = 1; s_accounts.getSection(acc); i++) {
+	    acc = item;
+	    acc << "_" << i;
+	}
+	setText("acc_account",acc,wnd);
 	for (const char** par = s_provParams; *par; par++) {
 	    String name;
 	    name << "acc_" << *par;
-	    setText(name,sect->getValue(*par));
+	    setText(name,sect->getValue(*par),wnd);
 	}
 	return true;
     }
@@ -1882,6 +1962,14 @@ void Client::enableAction(const ClientChannel* chan, const String& action)
 
 void Client::idleActions()
 {
+    s_debugMutex.lock();
+    String* log = s_debugLog;
+    s_debugLog = 0;
+    s_debugMutex.unlock();
+    if (log) {
+	addToLog(*log);
+	log->destruct();
+    }
     // arbitrary limit to let other threads run too
     for (int i = 0; i < 4; i++) {
 	if (!s_busy)
