@@ -95,7 +95,7 @@ using namespace TelEngine;
 #define CFG_SUFFIX ".conf"
 
 #define MAX_SANITY 5
-#define INIT_SANITY 10
+#define INIT_SANITY 30
 #define MAX_LOGBUFF 4096
 
 static u_int64_t s_nextinit = 0;
@@ -136,8 +136,8 @@ static void sighandler(int signal)
 String Engine::s_cfgpath(CFG_PATH);
 String Engine::s_cfgsuffix(CFG_SUFFIX);
 String Engine::s_modpath(MOD_PATH);
-String Engine::s_extramod;
 String Engine::s_modsuffix(DLL_SUFFIX);
+ObjList Engine::s_extramod;
 
 Engine::RunMode Engine::s_mode = Engine::Stopped;
 Engine* Engine::s_self = 0;
@@ -385,6 +385,9 @@ static int supervise(void)
 	    }
 	    ::fcntl(logfd[0],F_SETFL,O_NONBLOCK);
 	}
+	// reap any children we may have before spawning a new one
+	while (::waitpid(-1,0,WNOHANG) > 0)
+	    ;
 	s_childpid = ::fork();
 	if (s_childpid < 0) {
 	    int err = errno;
@@ -462,8 +465,6 @@ static int supervise(void)
 		::usleep(500000);
 	    }
 	    ::kill(s_childpid,SIGKILL);
-	    ::usleep(10000);
-	    ::waitpid(s_childpid,0,WNOHANG);
 	    s_childpid = -1;
 	}
 	if (s_logrotator) {
@@ -575,11 +576,34 @@ int Engine::run()
 #else
     ::signal(SIGPIPE,SIG_IGN);
 #endif
-    SysUsage::init();
-    s_runid = Time::secNow();
     s_cfg = configFile(s_cfgfile);
     s_cfg.load();
-    Debug(DebugAll,"Engine::run()");
+#ifdef _WINDOWS
+    int winTimerRes = s_cfg.getIntValue("general","wintimer");
+    if ((winTimerRes > 0) && (winTimerRes < 100)) {
+	typedef ULONG (__stdcall *NTSTR)(ULONG,BOOLEAN,PULONG);
+	String err;
+	HMODULE ntDll = GetModuleHandle("NTDLL.DLL");
+	if (ntDll) {
+	    NTSTR ntSTR = (NTSTR)GetProcAddress(ntDll,"NtSetTimerResolution");
+	    if (ntSTR) {
+		ULONG res = 0;
+		unsigned int ntstatus = ntSTR(10000*winTimerRes,true,&res);
+		if (ntstatus)
+		    err << "NTSTATUS " << ntstatus;
+	    }
+	    else
+		err = "NtSetTimerResolution not found";
+	}
+	else
+	    err = "NTDLL not found (Windows 9x or ME?)";
+	if (err)
+	    Debug(DebugWarn,"Timer resolution not set: %s",err.c_str());
+    }
+#endif
+    SysUsage::init();
+    s_runid = Time::secNow();
+    DDebug(DebugAll,"Engine::run()");
     install(new EngineStatusHandler);
     loadPlugins();
     Debug(DebugAll,"Loaded %d plugins",plugins.count());
@@ -597,7 +621,8 @@ int Engine::run()
     ::signal(SIGINT,sighandler);
     ::signal(SIGTERM,sighandler);
     Debug(DebugAll,"Engine dispatching start message");
-    dispatch("engine.start");
+    if (dispatch("engine.start"))
+	Debug(DebugGoOn,"Message engine.start was unexpectedly handled!");
     setStatus(SERVICE_RUNNING);
     long corr = 0;
 #ifndef _WINDOWS
@@ -622,8 +647,7 @@ int Engine::run()
 		else
 		    Debug(DebugWarn,"Unrecognized command '%s'",s->c_str());
 	    }
-	    s_cmds->destruct();
-	    s_cmds = 0;
+	    destruct(s_cmds);
 	}
 
 	if (s_init) {
@@ -690,7 +714,9 @@ int Engine::run()
     s_haltcode &= 0xff;
     Output("Yate engine is shutting down with code %d",s_haltcode);
     setStatus(SERVICE_STOP_PENDING);
-    dispatch("engine.halt");
+    ::signal(SIGINT,SIG_DFL);
+    if (dispatch("engine.halt"))
+	Debug(DebugGoOn,"Message engine.halt was unexpectedly handled!");
     checkPoint();
     Thread::msleep(200);
     m_dispatcher.dequeue();
@@ -700,7 +726,6 @@ int Engine::run()
     Thread::killall();
     checkPoint();
     m_dispatcher.dequeue();
-    ::signal(SIGINT,SIG_DFL);
     ::signal(SIGTERM,SIG_DFL);
 #ifndef _WINDOWS
     ::signal(SIGHUP,SIG_DFL);
@@ -841,9 +866,7 @@ void Engine::loadPlugins()
     const char *name = s_cfg.getValue("general","modpath");
     if (name)
 	s_modpath = name;
-    name = s_cfg.getValue("general","extrapath");
-    if (name)
-	s_extramod = name;
+    extraPath(s_cfg.getValue("general","extrapath"));
     s_maxworkers = s_cfg.getIntValue("general","maxworkers",s_maxworkers);
     s_restarts = s_cfg.getIntValue("general","restarts");
     m_dispatcher.warnTime(1000*(u_int64_t)s_cfg.getIntValue("general","warntime"));
@@ -857,8 +880,10 @@ void Engine::loadPlugins()
 	}
     }
     loadPluginDir(String::empty());
-    if (s_extramod)
-	loadPluginDir(s_extramod);
+    while (GenObject* extra = s_extramod.remove(false)) {
+	loadPluginDir(extra->toString());
+	extra->destruct();
+    }
     l = s_cfg.getSection("postload");
     if (l) {
         unsigned int len = l->length();
@@ -873,7 +898,8 @@ void Engine::loadPlugins()
 void Engine::initPlugins()
 {
     Output("Initializing plugins");
-    dispatch("engine.init");
+    if (dispatch("engine.init"))
+	Debug(DebugGoOn,"Message engine.init was unexpectedly handled!");
     ObjList *l = plugins.skipNull();
     for (; l; l = l->skipNext()) {
 	Plugin *p = static_cast<Plugin *>(l->get());
@@ -894,6 +920,13 @@ int Engine::usedPlugins()
     return used;
 }
 
+void Engine::extraPath(const String& path)
+{
+    if (path.null() || s_extramod.find(path))
+	return;
+    s_extramod.append(new String(path));
+}
+
 void Engine::halt(unsigned int code)
 {
     if (s_haltcode == -1)
@@ -968,6 +1001,7 @@ static void usage(bool client, FILE* f)
 "   -n configname  Use specified configuration name (%s)\n"
 "   -c pathname    Path to conf files directory (" CFG_PATH ")\n"
 "   -m pathname    Path to modules directory (" MOD_PATH ")\n"
+"   -x relpath     Relative path to extra modules directory (can be repeated)\n"
 "   -w directory   Change working directory\n"
 #ifdef RLIMIT_CORE
 "   -C             Enable core dumps if possible\n"
@@ -1154,6 +1188,14 @@ int Engine::main(int argc, const char** argv, const char** env, RunMode mode, bo
 			pc = 0;
 			workdir = argv[++i];
 			break;
+		    case 'x':
+			if (i+1 >= argc) {
+			    noarg(client,argv[i]);
+			    return ENOENT;
+			}
+			pc = 0;
+			extraPath(argv[++i]);
+			break;
 #ifdef RLIMIT_CORE
 		    case 'C':
 			s_coredump = true;
