@@ -67,6 +67,7 @@
 #endif
 
 #define MAX_SOCKLEN 1024
+#define MAX_RESWAIT 5000000
 
 using namespace TelEngine;
 
@@ -184,11 +185,14 @@ bool SocketAddr::host(const String& name)
 	    {
 		in_addr_t a = inet_addr(name);
 		if (a == INADDR_NONE) {
-		    s_mutex.lock();
-		    struct hostent* he = gethostbyname(name);
-		    if (he && (he->h_addrtype == AF_INET))
-			a = *((in_addr_t*)(he->h_addr_list[0]));
-		    s_mutex.unlock();
+		    if (s_mutex.lock(MAX_RESWAIT)) {
+			struct hostent* he = gethostbyname(name);
+			if (he && (he->h_addrtype == AF_INET))
+			    a = *((in_addr_t*)(he->h_addr_list[0]));
+			s_mutex.unlock();
+		    }
+		    else
+			Debug(DebugGoOn,"Resolver was busy, failing '%s'",name.c_str());
 		}
 		if (a != INADDR_NONE) {
 		    ((struct sockaddr_in*)m_address)->sin_addr.s_addr = a;
@@ -295,6 +299,34 @@ bool SocketAddr::supports(int family)
 }
 
 
+SocketFilter::SocketFilter()
+    : m_socket(0)
+{
+}
+
+SocketFilter::~SocketFilter()
+{
+    if (m_socket)
+	m_socket->removeFilter(this);
+}
+
+void* SocketFilter::getObject(const String& name) const
+{
+    if (name == "SocketFilter")
+	return const_cast<SocketFilter*>(this);
+    return GenObject::getObject(name);
+}
+
+void SocketFilter::timerTick(const Time& when)
+{
+}
+
+bool SocketFilter::valid() const
+{
+    return m_socket && m_socket->valid();
+}
+
+
 Stream::~Stream()
 {
 }
@@ -650,6 +682,7 @@ Socket::Socket(int domain, int type, int protocol)
 Socket::~Socket()
 {
     DDebug(DebugAll,"Socket::~Socket() handle=%d [%p]",m_handle,this);
+    clearFilters();
     terminate();
 }
 
@@ -891,6 +924,8 @@ bool Socket::getPeerName(SocketAddr& addr)
 
 int Socket::sendTo(const void* buffer, int length, const struct sockaddr* addr, socklen_t adrlen, int flags)
 {
+    if (!addr)
+	return send(buffer,length,flags);
     if (!buffer)
 	length = 0;
     int res = ::sendto(m_handle,(const char*)buffer,length,flags,addr,adrlen);
@@ -927,7 +962,10 @@ int Socket::recvFrom(void* buffer, int length, struct sockaddr* addr, socklen_t*
     if (adrlen && !addr)
 	*adrlen = 0;
     int res = ::recvfrom(m_handle,(char*)buffer,length,flags,addr,adrlen);
-    checkError(res,true);
+    if (checkError(res,true) && applyFilters(buffer,res,flags,addr,(adrlen ? *adrlen : 0))) {
+	m_error = EAGAIN;
+	res = socketError();
+    }
     return res;
 }
 
@@ -946,7 +984,10 @@ int Socket::recv(void* buffer, int length, int flags)
     if (!buffer)
 	length = 0;
     int res = ::recv(m_handle,(char*)buffer,length,flags);
-    checkError(res,true);
+    if (checkError(res,true) && applyFilters(buffer,res,flags)) {
+	m_error = EAGAIN;
+	res = socketError();
+    }
     return res;
 }
 
@@ -1108,4 +1149,49 @@ bool Socket::createPair(Socket& sock1, Socket& sock2, int domain)
     return false;
 }
 
+bool Socket::installFilter(SocketFilter* filter)
+{
+    if (!filter || filter->socket())
+	return false;
+    if (m_filters.find(filter))
+	return false;
+    filter->m_socket = this;
+    m_filters.append(filter);
+    return true;
+}
+
+void Socket::removeFilter(SocketFilter* filter, bool delobj)
+{
+    if (m_filters.remove(filter,delobj))
+	filter->m_socket = 0;
+}
+
+void Socket::clearFilters()
+{
+    m_filters.clear();
+}
+
+bool Socket::applyFilters(void* buffer, int length, int flags, const struct sockaddr* addr, socklen_t adrlen)
+{
+    if ((length <= 0) || !buffer)
+	return false;
+    if (!addr)
+	adrlen = 0;
+    for (ObjList* l = &m_filters; l; l = l->next()) {
+	SocketFilter* filter = static_cast<SocketFilter*>(l->get());
+	if (filter && filter->received(buffer,length,flags,addr,adrlen))
+	    return true;
+    }
+    return false;
+}
+
+void Socket::timerTick(const Time& when)
+{
+    for (ObjList* l = &m_filters; l; l = l->next()) {
+	SocketFilter* filter = static_cast<SocketFilter*>(l->get());
+	if (filter)
+	    filter->timerTick(when);
+    }
+}
+
 /* vi: set ts=8 sw=4 sts=4 noet: */
