@@ -64,7 +64,7 @@ public:
     CdrBuilder(const char *name);
     virtual ~CdrBuilder();
     void update(int type, u_int64_t val);
-    void update(const Message& msg, int type, u_int64_t val);
+    bool update(const Message& msg, int type, u_int64_t val);
     void emit(const char *operation = 0);
     String getStatus() const;
     static CdrBuilder* find(String &id);
@@ -77,7 +77,9 @@ private:
 	m_hangup;
     String m_dir;
     String m_status;
+    String m_cdrId;
     bool m_first;
+    bool m_write;
 };
 
 class Param : public String
@@ -99,6 +101,8 @@ static ObjList s_cdrs;
 static Mutex s_mutex;
 static ObjList s_params;
 static int s_res = 1;
+static int s_seq = 0;
+static String s_runId;
 
 // Time resolutions
 static TokenDict const s_timeRes[] = {
@@ -133,6 +137,9 @@ static const char* const s_forbidden[] = {
     "duration",
     "billtime",
     "ringtime",
+    "cdrwrite",
+    "cdrid",
+    "runid",
     0
 };
 
@@ -157,9 +164,11 @@ static const char* printTime(char* buf,u_int64_t usec)
 }
 
 CdrBuilder::CdrBuilder(const char *name)
-    : NamedList(name), m_dir("unknown"), m_status("unknown"), m_first(true)
+    : NamedList(name), m_dir("unknown"), m_status("unknown"),
+      m_first(true), m_write(true)
 {
     m_start = m_call = m_ringing = m_answer = m_hangup = 0;
+    m_cdrId = ++s_seq;
 }
 
 CdrBuilder::~CdrBuilder()
@@ -169,15 +178,16 @@ CdrBuilder::~CdrBuilder()
 
 void CdrBuilder::emit(const char *operation)
 {
+    if (null())
+	return;
     u_int64_t t_hangup = m_hangup ? m_hangup : Time::now();
 
     u_int64_t
-	t_start = m_start, t_call = m_call,
-	t_ringing = m_ringing, t_answer = m_answer;
-    if (!t_start)
-	t_start = t_call;
+	t_call = m_call, t_ringing = m_ringing, t_answer = m_answer;
+    if (!m_start)
+	m_start = t_call;
     if (!t_call)
-	t_call = t_start;
+	t_call = m_start;
     if (!t_ringing)
 	t_ringing = t_call;
     if (!t_answer)
@@ -194,11 +204,13 @@ void CdrBuilder::emit(const char *operation)
 
     char buf[64];
     Message *m = new Message("call.cdr");
-    m->addParam("time",printTime(buf,t_start));
+    m->addParam("time",printTime(buf,m_start));
     m->addParam("chan",c_str());
+    m->addParam("cdrid",m_cdrId);
+    m->addParam("runid",s_runId);
     m->addParam("operation",operation);
     m->addParam("direction",m_dir);
-    m->addParam("duration",printTime(buf,t_hangup - t_start));
+    m->addParam("duration",printTime(buf,t_hangup - m_start));
     m->addParam("billtime",printTime(buf,t_hangup - t_answer));
     m->addParam("ringtime",printTime(buf,t_answer - t_ringing));
     m->addParam("status",m_status);
@@ -211,6 +223,7 @@ void CdrBuilder::emit(const char *operation)
 	if (ext)
 	    m->setParam("external",ext);
     }
+    m->addParam("cdrwrite",String::boolText(m_write));
     unsigned int n = length();
     for (unsigned int i = 0; i < n; i++) {
 	const NamedString* s = getParam(i);
@@ -232,7 +245,8 @@ void CdrBuilder::update(int type, u_int64_t val)
 {
     switch (type) {
 	case CdrStart:
-	    m_start = val;
+	    if (!m_start)
+		m_start = val;
 	    break;
 	case CdrCall:
 	    m_call = val;
@@ -247,13 +261,27 @@ void CdrBuilder::update(int type, u_int64_t val)
 	    break;
 	case CdrHangup:
 	    m_hangup = val;
+	    s_mutex.lock();
 	    s_cdrs.remove(this);
+	    s_mutex.unlock();
 	    return;
     }
 }
 
-void CdrBuilder::update(const Message& msg, int type, u_int64_t val)
+bool CdrBuilder::update(const Message& msg, int type, u_int64_t val)
 {
+    if (type == CdrDrop) {
+	Debug("cdrbuild",DebugNote,"%s CDR for '%s'",
+	    (m_first ? "Dropping" : "Closing"),c_str());
+	// if we didn't generate an initialize generate no finalize
+	if (m_first)
+	    clear();
+	s_cdrs.remove(this);
+	return true;
+    }
+    // cdrwrite must be consistent over all emitted messages so we read it once
+    if (m_first)
+	m_write = msg.getBoolValue("cdrwrite",true);
     unsigned int n = msg.length();
     for (unsigned int i = 0; i < n; i++) {
 	const NamedString* s = msg.getParam(i);
@@ -290,9 +318,10 @@ void CdrBuilder::update(const Message& msg, int type, u_int64_t val)
 
     if (type == CdrHangup) {
 	s_cdrs.remove(this);
-	return;
+	return false;
     }
     emit();
+    return false;
 }
 
 CdrBuilder* CdrBuilder::find(String &id)
@@ -311,6 +340,10 @@ bool CdrHandler::received(Message &msg)
     if (!msg.getBoolValue("cdrtrack",true))
 	return false;
     String id(msg.getValue("id"));
+    if (m_type == CdrDrop) {
+	if (!id.startSkip("cdrbuild/",false))
+	    return false;
+    }
     if (id.null()) {
 	id = msg.getValue("module");
 	id += "/";
@@ -320,13 +353,16 @@ bool CdrHandler::received(Message &msg)
 	if (id == "//")
 	    return false;
     }
+    bool rval = false;
+    s_mutex.lock();
     CdrBuilder *b = CdrBuilder::find(id);
     if (!b && ((m_type == CdrStart) || (m_type == CdrCall))) {
 	b = new CdrBuilder(id);
 	s_cdrs.append(b);
     }
+    s_mutex.unlock();
     if (b)
-	b->update(msg,m_type,msg.msgTime().usec());
+	rval = b->update(msg,m_type,msg.msgTime().usec());
     else
 	Debug("CdrBuilder",DebugInfo,"Got message '%s' for untracked id '%s'",
 	    msg.c_str(),id.c_str());
@@ -337,7 +373,7 @@ bool CdrHandler::received(Message &msg)
 	    b->emit();
 	}
     }
-    return false;
+    return rval;
 };
 		    
 bool StatusHandler::received(Message &msg)
@@ -417,13 +453,14 @@ void CdrBuildPlugin::initialize()
     s_mutex.unlock();
     if (m_first) {
 	m_first = false;
+	s_runId = Engine::runId();
 	Engine::install(new CdrHandler("chan.startup",CdrStart));
 	Engine::install(new CdrHandler("call.route",CdrRoute));
 	Engine::install(new CdrHandler("call.execute",CdrCall));
 	Engine::install(new CdrHandler("call.ringing",CdrRinging));
 	Engine::install(new CdrHandler("call.answered",CdrAnswer));
 	Engine::install(new CdrHandler("chan.hangup",CdrHangup));
-	Engine::install(new CdrHandler("call.dropcdr",CdrDrop));
+	Engine::install(new CdrHandler("call.drop",CdrDrop));
 	Engine::install(new CdrHandler("engine.halt",EngHalt,150));
 	Engine::install(new StatusHandler);
     }
