@@ -82,7 +82,7 @@ static int s_roomAlloc = 0;
 class ConfRoom : public DataSource
 {
 public:
-    virtual ~ConfRoom();
+    virtual void destroyed();
     static ConfRoom* get(const String& name, const NamedList* params = 0);
     virtual const String& toString() const
 	{ return m_name; }
@@ -97,7 +97,7 @@ public:
     inline ConfChan* recorder() const
 	{ return m_record; }
     void mix(ConfConsumer* cons = 0);
-    void addChannel(ConfChan* chan);
+    void addChannel(ConfChan* chan, bool player = false);
     void delChannel(ConfChan* chan);
     bool setRecording(const NamedList& params);
 private:
@@ -105,6 +105,7 @@ private:
     String m_name;
     ObjList m_chans;
     String m_notify;
+    String m_playerId;
     bool m_lonely;
     ConfChan* m_record;
     int m_rate;
@@ -227,9 +228,9 @@ ConfRoom* ConfRoom::get(const String& name, const NamedList* params)
     Lock lock(&__plugin);
     ObjList* l = s_rooms.find(name);
     ConfRoom* room = l ? static_cast<ConfRoom*>(l->get()) : 0;
-    if (room)
-	room->ref();
-    else if (params)
+    if (room && !room->ref())
+	room = 0;
+    if (params && !room)
 	room = new ConfRoom(name,*params);
     return room;
 }
@@ -263,9 +264,9 @@ ConfRoom::ConfRoom(const String& name, const NamedList& params)
     }
 }
 
-ConfRoom::~ConfRoom()
+void ConfRoom::destroyed()
 {
-    DDebug(&__plugin,DebugAll,"ConfRoom::~ConfRoom() '%s' [%p]",m_name.c_str(),this);
+    DDebug(&__plugin,DebugAll,"ConfRoom::destroyed() '%s' [%p]",m_name.c_str(),this);
     // plugin must be locked as the destructor is called when room is dereferenced
     Lock lock(&__plugin);
     s_rooms.remove(this,false);
@@ -277,23 +278,29 @@ ConfRoom::~ConfRoom()
 	m->addParam("room",m_name);
 	Engine::enqueue(m);
     }
+    DataSource::destroyed();
 }
 
 // Add one channel to the room
-void ConfRoom::addChannel(ConfChan* chan)
+void ConfRoom::addChannel(ConfChan* chan, bool player)
 {
     if (!chan)
 	return;
     m_chans.append(chan);
+    if (player)
+	m_playerId = chan->id();
     if (!chan->isUtility()) {
 	m_users++;
 	if (m_notify) {
 	    String tmp(m_users);
 	    Message* m = new Message("chan.notify");
+	    m->addParam("id",chan->id());
 	    m->addParam("targetid",m_notify);
 	    m->addParam("event","joined");
 	    m->addParam("room",m_name);
 	    m->addParam("users",tmp);
+	    if (m_playerId)
+		m->addParam("player",m_playerId);
 	    Engine::enqueue(m);
 	}
     }
@@ -304,20 +311,28 @@ void ConfRoom::delChannel(ConfChan* chan)
 {
     if (!chan)
 	return;
+    Lock lock(mutex());
     if (chan == m_record)
 	m_record = 0;
+    if (m_playerId && (chan->id() == m_playerId))
+	m_playerId.clear();
     if (m_chans.remove(chan,false) && !chan->isUtility()) {
 	m_users--;
 	bool alone = (m_users == 1);
+	lock.drop();
 	if (m_notify) {
 	    String tmp(m_users);
 	    Message* m = new Message("chan.notify");
+	    m->addParam("id",chan->id());
 	    m->addParam("targetid",m_notify);
 	    m->addParam("event","left");
 	    m->addParam("room",m_name);
 	    m->addParam("users",tmp);
-	    // easy to check parameter indicating one user was left alone
-	    m->addParam("lonely",String::boolText(alone));
+	    // easy to check parameter indicating one user will be left alone
+	    if (m_lonely)
+		m->addParam("lonely",String::boolText(alone));
+	    if (m_playerId)
+		m->addParam("player",m_playerId);
 	    Engine::enqueue(m);
 	}
 
@@ -608,7 +623,7 @@ ConfChan::ConfChan(const String& name, const NamedList& params)
     m_room = ConfRoom::get(name,&params);
     if (m_room) {
 	m_address = name;
-	m_room->addChannel(this);
+	m_room->addChannel(this,params.getBoolValue("player",false));
 	RefPointer<ConfConsumer> cons;
 	if (voice) {
 	    cons = new ConfConsumer(m_room,smart);
@@ -659,12 +674,13 @@ ConfChan::~ConfChan()
 {
     DDebug(this,DebugAll,"ConfChan::~ConfChan() %s [%p]",id().c_str(),this);
     Lock lock(&__plugin);
-    // first make sure we don't pump any more data
-    setConsumer();
-    setSource();
-    // then remove ourselves from the room
-    if (m_room)
-	m_room->delChannel(this);
+    // keep the room referenced until we are done
+    RefPointer<ConfRoom> room = m_room;
+    // remove ourselves from the room's mixer
+    if (room)
+	room->delChannel(this);
+    // now we can safely remove the data streams
+    clearEndpoint();
     if (m_billing)
 	Engine::enqueue(message("chan.hangup"));
 }
