@@ -35,6 +35,7 @@ namespace { // anonymous
 #define ENUM_DEF_TIMEOUT 3
 #define ENUM_DEF_RETRIES 2
 #define ENUM_DEF_MINLEN  8
+#define ENUM_DEF_MAXCALL 30000
 
 class NAPTR : public GenObject
 {
@@ -67,6 +68,9 @@ public:
 	: MessageHandler("call.route",prio)
 	{ }
     virtual bool received(Message& msg);
+private:
+    static bool resolve(Message& msg,bool canRedirect);
+    static void addRoute(String& dest,const String& src);
 };
 
 class EnumModule : public Module
@@ -104,17 +108,22 @@ static int dn_string(const unsigned char* end, const unsigned char* src, char *d
 
 
 static String s_prefix;
-static String s_domain;
-static String s_backup;
+static String s_forkStop;
+static String s_domains;
 static unsigned int s_minlen;
 static int s_timeout;
 static int s_retries;
+static int s_maxcall;
 
 static bool s_redirect;
+static bool s_autoFork;
 static bool s_sipUsed;
 static bool s_iaxUsed;
 static bool s_h323Used;
+static bool s_xmppUsed;
 static bool s_telUsed;
+static bool s_voiceUsed;
+static bool s_pstnUsed;
 static bool s_voidUsed;
 
 static Mutex s_mutex;
@@ -226,6 +235,7 @@ static ObjList* naptrQuery(const char* dname)
     return lst;
 }
 
+
 NAPTR::NAPTR(int ord, int pref, const char* flags, const char* serv, const char* regexp, const char* replace)
     : m_order(ord), m_pref(pref), m_flags(flags), m_service(serv), m_replace(replace)
 {
@@ -257,15 +267,25 @@ bool NAPTR::replace(String& str)
 }
 
 
+// Routing message handler, performs checks and calls resolve method
 bool EnumHandler::received(Message& msg)
 {
-    if (s_domain.null() && s_backup.null())
+    if (s_domains.null() || !msg.getBoolValue("enumroute",true))
 	return false;
+    // perform per-thread initialization of resolver and timeout settings
+    if (!resolvInit())
+	return false;
+    return resolve(msg,s_telUsed);
+}
+
+// Resolver function, may call itself recursively at most once
+bool EnumHandler::resolve(Message& msg,bool canRedirect)
+{
     // give preference to full (e164) called number if exists
     String called(msg.getValue("calledfull"));
     if (called.null())
 	called = msg.getValue("called");
-    if (called.null() && msg.getBoolValue("enumroute",true))
+    if (called.null())
 	return false;
     // check if the called starts with international prefix, remove it
     if (!(called.startSkip("+",false) ||
@@ -273,8 +293,10 @@ bool EnumHandler::received(Message& msg)
 	return false;
     if (called.length() < s_minlen)
 	return false;
-    // perform per-thread initialization of resolver and timeout settings
-    if (!resolvInit())
+    s_mutex.lock();
+    ObjList* domains = s_domains.split(',',false);
+    s_mutex.unlock();
+    if (!domains)
 	return false;
     bool rval = false;
     // put the standard international prefix in front
@@ -284,18 +306,25 @@ bool EnumHandler::received(Message& msg)
 	tmp << called.at(i) << ".";
     u_int64_t dt = Time::now();
     ObjList* res = 0;
-    if (s_domain)
-	res = naptrQuery(tmp + s_domain);
-    if (s_backup && !res)
-	res = naptrQuery(tmp + s_backup);
+    for (ObjList* l = domains; l; l = l->next()) {
+	const String* s = static_cast<const String*>(l->get());
+	if (!s || s->null())
+	    continue;
+	res = naptrQuery(tmp + *s);
+	if (res)
+	    break;
+    }
     dt = Time::now() - dt;
     Debug(&emodule,DebugInfo,"Returned %d NAPTR records in %u.%06u s",
 	res ? res->count() : 0,
 	(unsigned int)(dt / 1000000),
 	(unsigned int)(dt % 1000000));
+    domains->destruct();
     bool reroute = false;
     bool unassigned = false;
     if (res) {
+	msg.retValue().clear();
+	bool autoFork = msg.getBoolValue("autofork",s_autoFork);
 	ObjList* cur = res;
 	for (; cur; cur = cur->next()) {
 	    NAPTR* ptr = static_cast<NAPTR*>(cur->get());
@@ -305,43 +334,84 @@ bool EnumHandler::received(Message& msg)
 		ptr->order(),ptr->pref(),ptr->serv().c_str());
 	    String serv = ptr->serv();
 	    serv.toUpper();
-	    if (s_sipUsed && (serv == "E2U+SIP") && ptr->replace(called)) {
-		msg.retValue() = "sip/" + called;
+	    String callto = called;
+	    if (s_sipUsed && (serv == "E2U+SIP") && ptr->replace(callto)) {
+		addRoute(msg.retValue(),"sip/" + callto);
 		rval = true;
+		if (autoFork)
+		    continue;
 		break;
 	    }
-	    if (s_iaxUsed && (serv == "E2U+IAX2") && ptr->replace(called)) {
-		msg.retValue() = "iax/" + called;
+	    if (s_iaxUsed && (serv == "E2U+IAX2") && ptr->replace(callto)) {
+		addRoute(msg.retValue(),"iax/" + callto);
 		rval = true;
+		if (autoFork)
+		    continue;
 		break;
 	    }
-	    if (s_h323Used && (serv == "E2U+H323") && ptr->replace(called)) {
-		msg.retValue() = "h323/" + called;
+	    if (s_h323Used && (serv == "E2U+H323") && ptr->replace(callto)) {
+		addRoute(msg.retValue(),"h323/" + callto);
 		rval = true;
+		if (autoFork)
+		    continue;
 		break;
 	    }
-	    if (s_telUsed && (serv == "E2U+TEL") && ptr->replace(called)) {
-		if (called.startSkip("tel:",false) ||
-		    called.startSkip("TEL:",false) ||
-		    called.startSkip("e164:",false) ||
-		    called.startSkip("E164:",false))
+	    if (s_xmppUsed && (serv == "E2U+XMPP") && ptr->replace(callto)) {
+		addRoute(msg.retValue(),"jingle/" + callto);
+		rval = true;
+		if (autoFork)
+		    continue;
+		break;
+	    }
+	    if (s_pstnUsed && serv.startsWith("E2U+PSTN") && ptr->replace(callto)) {
+		addRoute(msg.retValue(),"pstn/" + callto);
+		rval = true;
+		if (autoFork)
+		    continue;
+		break;
+	    }
+	    if (s_voiceUsed && serv.startsWith("E2U+VOICE") && ptr->replace(callto)) {
+		addRoute(msg.retValue(),"voice/" + callto);
+		rval = true;
+		if (autoFork)
+		    continue;
+		break;
+	    }
+	    if (canRedirect && (serv == "E2U+TEL") && ptr->replace(callto)) {
+		if (callto.startSkip("tel:",false) ||
+		    callto.startSkip("TEL:",false) ||
+		    callto.startSkip("e164:",false) ||
+		    callto.startSkip("E164:",false))
 		{
 		    reroute = true;
-		    msg.setParam("called",called);
+		    rval = false;
+		    msg.setParam("called",callto);
 		    msg.clearParam("calledfull");
+		    if (msg.retValue()) {
+			Debug(&emodule,DebugMild,"Redirect drops collected route: %s",
+			    msg.retValue().c_str());
+			msg.retValue().clear();
+		    }
 		    break;
 		}
+		continue;
 	    }
-	    if (s_voidUsed && serv.startsWith("E2U+VOID") && ptr->replace(called)) {
+	    if (s_voidUsed && serv.startsWith("E2U+VOID") && ptr->replace(callto)) {
 		// remember it's unassigned but still continue scanning
 		unassigned = true;
 	    }
 	}
 	res->destruct();
     }
-    if (rval && s_redirect)
-	msg.setParam("redirect",String::boolText(true));
     s_mutex.lock();
+    if (rval) {
+	if (msg.retValue().startsWith("fork",true)) {
+	    msg.setParam("maxcall",String(s_maxcall));
+	    msg.setParam("fork.stop",s_forkStop);
+	}
+        else if (s_redirect)
+	    msg.setParam("redirect",String::boolText(true));
+    }
     s_queries++;
     if (rval)
 	s_routed++;
@@ -349,6 +419,8 @@ bool EnumHandler::received(Message& msg)
 	s_reroute++;
     emodule.changed();
     s_mutex.unlock();
+    if (reroute)
+	return resolve(msg,false);
     if (unassigned && !rval) {
 	rval = true;
 	msg.retValue() = "-";
@@ -357,6 +429,19 @@ bool EnumHandler::received(Message& msg)
     return rval;
 }
 
+// Add one route to the result, take care of forking
+void EnumHandler::addRoute(String& dest,const String& src)
+{
+    if (dest.null())
+	dest = src;
+    else {
+	if (!dest.startsWith("fork",true))
+	    dest = "fork " + dest;
+	dest << " | " << src;
+    }
+}
+
+
 void EnumModule::statusParams(String& str)
 {
     str.append("queries=",",") << s_queries << ",routed=" << s_routed << ",rerouted=" << s_reroute;
@@ -377,11 +462,19 @@ void EnumModule::initialize()
     if ((prio <= 0) && !m_init)
 	return;
     Output("Initializing ENUM routing");
+    s_mutex.lock();
     // in most of the world this default international prefix should work
     s_prefix = cfg.getValue("general","prefix","00");
+    s_domains = cfg.getValue("general","domains");
+    if (s_domains.null()) {
+	// old style, just for compatibility
+	s_domains = cfg.getValue("general","domain","e164.arpa");
+	s_domains.append(cfg.getValue("general","backup","e164.org"),",");
+    }
+    s_forkStop = cfg.getValue("general","forkstop","busy");
+    s_mutex.unlock();
+    DDebug(&emodule,DebugInfo,"Domain list: %s",s_domains.c_str());
     s_minlen = cfg.getIntValue("general","minlen",ENUM_DEF_MINLEN);
-    s_domain = cfg.getValue("general","domain","e164.arpa");
-    s_backup = cfg.getValue("general","backup","e164.org");
     int tmp = cfg.getIntValue("general","timeout",ENUM_DEF_TIMEOUT);
     // limit between 1 and 10 seconds
     if (tmp < 1)
@@ -398,13 +491,26 @@ void EnumModule::initialize()
     s_retries = tmp;
     // overall a resolve attempt will take at most 50s per domain
 
+    tmp = cfg.getIntValue("general","maxcall",ENUM_DEF_MAXCALL);
+    // limit between 2 and 120 seconds
+    if (tmp < 2000)
+	tmp = 2000;
+    if (tmp > 120000)
+	tmp = 120000;
+    s_maxcall = tmp;
+
     s_redirect = cfg.getBoolValue("general","redirect");
-    s_sipUsed = cfg.getBoolValue("protocols","sip",true);
-    s_iaxUsed = cfg.getBoolValue("protocols","iax",true);
+    s_autoFork = cfg.getBoolValue("general","autofork");
+    s_sipUsed  = cfg.getBoolValue("protocols","sip",true);
+    s_iaxUsed  = cfg.getBoolValue("protocols","iax",true);
     s_h323Used = cfg.getBoolValue("protocols","h323",true);
+    s_xmppUsed = cfg.getBoolValue("protocols","jingle",true);
     s_voidUsed = cfg.getBoolValue("protocols","void",true);
     // by default don't support the number rerouting
-    s_telUsed = cfg.getBoolValue("protocols","tel",false);
+    s_telUsed  = cfg.getBoolValue("protocols","tel",false);
+    // also don't enable gateways by default as more setup is needed
+    s_pstnUsed = cfg.getBoolValue("protocols","pstn",false);
+    s_voiceUsed= cfg.getBoolValue("protocols","voice",false);
     if (m_init || (prio <= 0))
 	return;
     m_init = true;
