@@ -250,7 +250,7 @@ private:
 class ExtModHandler : public MessageHandler
 {
 public:
-    ExtModHandler(const char *name) : MessageHandler(name) { }
+    ExtModHandler(const char *name, unsigned prio) : MessageHandler(name,prio) { }
     virtual bool received(Message &msg);
 };
 
@@ -726,10 +726,7 @@ bool ExtModReceiver::start()
 
 bool ExtModReceiver::flush()
 {
-    if (m_watcher) {
-	m_watcher->destruct();
-	m_watcher = 0;
-    }
+    TelEngine::destruct(m_watcher);
     // Make sure we release all pending messages and not accept new ones
     if (!Engine::exiting())
 	m_relays.clear();
@@ -1136,7 +1133,14 @@ bool ExtModReceiver::processLine(const char* line)
 	    m_relays.append(r);
 	    Engine::install(r);
 	}
-	Debug("ExtModReceiver",DebugAll,"Install '%s', prio %d %s", id.c_str(),prio,ok ? "ok" : "failed");
+	if (debugAt(DebugAll)) {
+	    String tmp;
+	    if (fname)
+		tmp << "filter: '" << fname << "'='" << fvalue << "' ";
+	    tmp << (ok ? "ok" : "failed");
+	    Debug("ExtModReceiver",DebugAll,"Install '%s', prio %d %s",
+		id.c_str(),prio,tmp.c_str());
+	}
 	String out("%%<install:");
 	out << prio << ":" << id << ":" << ok;
 	outputLine(out);
@@ -1347,26 +1351,30 @@ bool ExtModCommand::received(Message& msg)
 	return true;
     }
     int blank = line.find(' ');
-    if (blank <= 0) {
-	if (line == "stop")
-	    return false;
-	ExtModReceiver *r = ExtModReceiver::build(line,(const char*)0);
-	msg.retValue() = r ? "External start attempt\r\n" : "External command failed!\r\n";
-	return true;
-    }
-    if (line.startSkip("stop")) {
+    bool start = line.startSkip("start");
+    bool restart = start || line.startSkip("restart");
+    if (restart || line.startSkip("stop")) {
 	if (line.null())
 	    return false;
-	ExtModReceiver *r = ExtModReceiver::find(line);
+	blank = line.find(' ');
+	ExtModReceiver *r = ExtModReceiver::find(line.substr(0,blank));
 	if (r) {
-	    r->die();
-	    msg.retValue() = "External command stopped\r\n";
+	    if (start) {
+		msg.retValue() = "External already running\r\n";
+		return true;
+	    }
+	    else {
+		r->die();
+		msg.retValue() = "External command stopped\r\n";
+	    }
 	}
 	else
 	    msg.retValue() = "External not running\r\n";
-	return true;
+	if (!restart)
+	    return true;
     }
-    ExtModReceiver *r = ExtModReceiver::build(line.substr(0,blank),line.substr(blank+1));
+    ExtModReceiver *r = ExtModReceiver::build(line.substr(0,blank),
+	(blank >= 0) ? line.substr(blank+1).c_str() : (const char*)0);
     msg.retValue() = r ? "External start attempt\r\n" : "External command failed\r\n";
     return true;
 }
@@ -1498,7 +1506,7 @@ void ExtModulePlugin::initialize()
     s_timeout = s_cfg.getIntValue("general","timeout",MSG_TIMEOUT);
     s_timebomb = s_cfg.getBoolValue("general","timebomb",false);
     if (!m_handler) {
-	m_handler = new ExtModHandler("call.execute");
+	m_handler = new ExtModHandler("call.execute",s_cfg.getIntValue("general","priority",100));
 	Engine::install(m_handler);
 	Engine::install(new ExtModCommand("engine.command"));
 	NamedList *sect = 0;
