@@ -533,7 +533,7 @@ FaxChan::FaxChan(bool outgoing, const char *file, bool sender, Message& msg)
     m_caller = msg.getBoolValue("faxcaller",!outgoing);
     m_ecm = msg.getBoolValue("faxecm");
     m_address = file;
-    Engine::enqueue(message("chan.startup"));
+    Engine::enqueue(message("chan.startup",msg));
 }
 
 // Destructor - clears all (audio, image) endpoints early
@@ -657,7 +657,7 @@ bool FaxDriver::setDebug(Message& msg, const String& target)
 {
     if (target == "spandsp") {
 	s_debug = msg.getBoolValue("line",s_debug);
-	msg.retValue() << "Detailed spandsp debugging " << (s_debug ? "on" : "off") << "\n";
+	msg.retValue() << "Detailed spandsp debugging " << (s_debug ? "on" : "off") << "\r\n";
 	return true;
     }
     return Driver::setDebug(msg,target);
