@@ -756,8 +756,8 @@ void PriChan::restart(bool outgoing)
 void PriChan::closeData()
 {
     m_span->lock();
-    setSource();
-    setConsumer();
+    // remove all types of data nodes
+    clearEndpoint();
     m_span->unlock();
 }
 
@@ -815,6 +815,8 @@ void PriChan::hangup(int cause)
     if (inUse())
 	Debug(this,DebugInfo,"Hanging up %s (%s) in state %s: %s (%d)",
 	    id().c_str(),address().c_str(),chanStatus(),reason,cause);
+    timeout(0);
+    maxcall(0);
     m_timeout = 0;
     m_targetid.clear();
     disconnect(lookup(cause,dict_str2cause,reason));
@@ -830,6 +832,7 @@ void PriChan::hangup(int cause)
 	m->addParam("reason",pri_cause2str(cause));
 	Engine::enqueue(m);
     }
+    m_answered = false;
     m_billid.clear();
     status(chanStatus());
 }
@@ -871,8 +874,18 @@ void PriChan::gotDigits(const char *digits, bool overlapped)
 
 void PriChan::sendDigit(char digit)
 {
-    if (m_call)
-	::pri_information(m_span->pri(),m_call,digit);
+    if (!m_call)
+	return;
+#ifdef PRI_KEYPAD_FACILITY_TX
+    if (isAnswered()) {
+	char buf[2];
+	buf[0] = digit;
+	buf[1] = '\0';
+	::pri_keypad_facility(m_span->pri(),m_call,buf);
+	return;
+    }
+#endif
+    ::pri_information(m_span->pri(),m_call,digit);
 }
 
 bool PriChan::call(Message &msg, const char *called)
@@ -941,7 +954,7 @@ bool PriChan::call(Message &msg, const char *called)
     setTimeout(30000000);
     status(chanStatus());
     setMaxcall(msg);
-    Message *m = message("chan.startup");
+    Message* m = message("chan.startup",msg);
     m->setParam("caller",msg.getValue("caller"));
     m->setParam("called",msg.getValue("called"));
     m->setParam("billid",msg.getValue("billid"));
@@ -973,7 +986,7 @@ void PriChan::ring(pri_event_ring &ev)
     // we signal ringing without media if the library doesn't know any better
     ::pri_acknowledge(m_span->pri(),m_call,m_chan,0);
 #endif
-    Message *m = message("chan.startup");
+    Message* m = message("chan.startup");
     m->addParam("span",String(m_span->span()));
     m->addParam("channel",String(m_chan));
     m->addParam("direction","incoming");
