@@ -120,7 +120,8 @@ private:
 };
 
 MOHSource::MOHSource(const String &name, const String &command_line)
-    :   ThreadedSource("slin"), m_name(name), m_command_line(command_line), m_swap(false), m_brate(16000)
+    : ThreadedSource("slin"),
+      m_name(name), m_command_line(command_line), m_swap(false), m_brate(16000)
 {
     Debug(DebugAll,"MOHSource::MOHSource(\"%s\", \"%s\") [%p]", name.c_str(), command_line.c_str(), this);
 }
@@ -153,7 +154,7 @@ MOHSource *MOHSource::getSource(const String &name)
     cmd = s_cfg.getValue("mohs", name);
     if (cmd) {
 	MOHSource *s = new MOHSource(name, cmd);
-	if(s->start()) {
+	if (s->start("MOHSource")) {
 	    sources.append(s);
 	    return s;
 	} else
@@ -309,32 +310,41 @@ bool MOHHandler::received(Message &msg)
 	}
     }
     else {
-	const char *targ = msg.getValue("target");
-	if (!targ) {
-	    Debug(DebugWarn,"MOH outgoing call with no target!");
-	    return false;
-	}
-	Message m("call.route");
-	m.addParam("id",dest);
-	m.addParam("caller",dest);
-	m.addParam("called",targ);
-	if (Engine::dispatch(m)) {
-	    m = "call.execute";
-	    m.addParam("callto",m.retValue());
-	    m.retValue() = 0;
-	    MOHChan *mc = new MOHChan(dest.matchString(1).c_str());
-	    m.setParam("id",mc->id());
-	    m.userData(mc);
-	    if (Engine::dispatch(m)) {
-		msg.setParam("id",mc->id());
-		mc->deref();
-		return true;
+	String callto(msg.getValue("direct"));
+	Message m(msg);
+	m.retValue().clear();
+	m.clearParam("callto");
+	m.setParam("id",dest);
+	m.setParam("caller",dest);
+	if (callto.null()) {
+	    m = "call.route";
+	    const char *targ = msg.getValue("target");
+	    if (!targ)
+		targ = msg.getValue("called");
+	    if (!targ) {
+		Debug(DebugWarn,"MOH outgoing call with no target!");
+		return false;
 	    }
-	    Debug(DebugWarn,"MOH outgoing call not accepted!");
-	    mc->destruct();
+	    m.setParam("called",targ);
+	    if (!Engine::dispatch(m))
+		return false;
+	    callto = m.retValue();
+	    if (callto.null() || (callto == "-"))
+		return false;
+	    m.retValue().clear();
 	}
-	else
-	    Debug(DebugWarn,"MOH outgoing call but no route!");
+	m = "call.execute";
+	m.addParam("callto",callto);
+	MOHChan *mc = new MOHChan(dest.matchString(1).c_str());
+	m.setParam("id",mc->id());
+	m.userData(mc);
+	if (Engine::dispatch(m)) {
+	    msg.setParam("id",mc->id());
+	    mc->deref();
+	    return true;
+	}
+	Debug(DebugWarn,"MOH outgoing call not accepted!");
+	mc->destruct();
 	return false;
     }
     return true;
