@@ -45,6 +45,7 @@ public:
 protected:
     bool errorBeep(const char* source = 0);
     void setGuest(const Message& msg);
+    void setParams(const Message& msg);
     u_int64_t m_last;
     bool m_pass;
     bool m_guest;
@@ -115,6 +116,20 @@ static String s_error;
 
 static Configuration s_cfg;
 
+// Utility function - copy parameters requested in "copyparams" parameter
+static void copyParams(NamedList& dest, const NamedList& original, const NamedList* pbxkeep = 0)
+{
+    const String* params = original.getParam("copyparams");
+    if (params && *params)
+	dest.copyParams(original,*params);
+    if (pbxkeep) {
+	params = original.getParam("pbxparams");
+	if (params && *params)
+	    dest.copyParams(*pbxkeep,*params);
+    }
+}
+
+
 ChanAssist* PBXList::create(Message& msg, const String& id)
 {
     if (msg == "chan.startup" || msg.userObject("Channel")) {
@@ -221,6 +236,7 @@ bool PBXAssist::msgDisconnect(Message& msg, const String& reason)
 	    m->addParam("pbxstate",m_state);
 	    m->copyParam(m_keep,"billid");
 	    m->copyParam(m_keep,"caller");
+	    copyParams(*m,msg,&m_keep);
 	    if (isE164(called)) {
 		// divert target is a number so we have to route it
 		m->addParam("called",called);
@@ -228,7 +244,7 @@ bool PBXAssist::msgDisconnect(Message& msg, const String& reason)
 		*m = "call.route";
 		if (!Engine::dispatch(m) || m->retValue().null() || (m->retValue() == "-") || (m->retValue() == "error")) {
 		    // routing failed
-		    delete m;
+		    TelEngine::destruct(m);
 		    return errorBeep();
 		}
 		called = m->retValue();
@@ -332,6 +348,18 @@ bool PBXAssist::msgTone(Message& msg)
 	if (tmp) {
 	    Debug(list(),DebugNote,"Chan '%s' triggered operation '%s' in state '%s' holding '%s'",
 		id().c_str(),tmp,m_state.c_str(),m_peer1.c_str());
+	    // we need special handling for transparent passing of keys
+	    if (String(tmp) == "transparent") {
+		String keys = sect->getValue("text");
+		if (keys) {
+		    keys = m_tones.replaceMatches(keys);
+		    msg.replaceParams(keys);
+		    msg.setParam("text",keys);
+		}
+		m_tones = sect->getValue("pastekeys");
+		// let the DTMF message pass through
+		return false;
+	    }
 	    Message* m = new Message("chan.masquerade");
 	    m->addParam("id",id());
 	    m->addParam("message",sect->getValue("message","chan.operation"));
@@ -387,6 +415,14 @@ void PBXAssist::setGuest(const Message& msg)
     }
 }
 
+// Copy extra PBX parameters from message
+void PBXAssist::setParams(const Message& msg)
+{
+    const String* params = msg.getParam("pbxparams");
+    if (params && *params)
+	m_keep.copyParams(msg,*params);
+}
+
 // Operation message handler - calls one of the operation handlers
 bool PBXAssist::msgOperation(Message& msg, const String& operation)
 {
@@ -432,6 +468,7 @@ bool PBXAssist::operSetState(Message& msg)
     m_state = msg.getValue("state",m_state);
     m_room = msg.getValue("room",m_room);
     setGuest(msg);
+    setParams(msg);
     return true;
 }
 
@@ -469,6 +506,7 @@ bool PBXAssist::operConference(Message& msg)
 	if (room)
 	    m.addParam("room",room);
 	m.addParam("pbxstate",m_state);
+	copyParams(m,msg);
 
 	if (Engine::dispatch(m) && m.userData()) {
 	    m_room = m.getParam("room");
@@ -478,6 +516,7 @@ bool PBXAssist::operConference(Message& msg)
 		m2->addParam("id",m_peer1);
 		m2->addParam("message","call.execute");
 		m2->addParam("callto",m_room);
+		copyParams(*m2,msg);
 		Engine::enqueue(m2);
 		// also set held peer's PBX state if it has one
 		m2 = new Message("chan.operation");
@@ -502,6 +541,7 @@ bool PBXAssist::operConference(Message& msg)
 	Message* m = c->message("call.execute",false,true);
 	m->addParam("callto",room);
 	m->addParam("pbxstate",m_state);
+	copyParams(*m,msg);
 	Engine::enqueue(m);
     }
 
@@ -528,11 +568,12 @@ bool PBXAssist::operSecondCall(Message& msg)
     m->copyParam(m_keep,"caller");
     m->addParam("called",msg.getValue("target"));
     m->addParam("pbxstate",m_state);
+    copyParams(*m,msg,&m_keep);
     // no error check as handling preroute is optional
     Engine::dispatch(m);
     *m = "call.route";
     if (!Engine::dispatch(m) || m->retValue().null() || (m->retValue() == "-") || (m->retValue() == "error")) {
-	delete m;
+	TelEngine::destruct(m);
 	return errorBeep();
     }
     m_state = "call";
@@ -575,6 +616,7 @@ bool PBXAssist::operOnHold(Message& msg)
 	m->addParam("callto","tone/dial");
 	m->addParam("message","call.execute");
 	m->addParam("reason","hold");
+	copyParams(*m,msg);
 	m_state = "dial";
     }
     m->addParam("pbxstate",m_state);
@@ -607,6 +649,7 @@ bool PBXAssist::operReturnConf(Message& msg)
     Message* m = c->message("call.execute",false,true);
     m->addParam("callto",m_room);
     m->addParam("pbxstate",m_state);
+    copyParams(*m,msg);
     Engine::enqueue(m);
     return true;
 }
@@ -620,6 +663,7 @@ bool PBXAssist::operReturnTone(Message& msg)
     m->addParam("callto","tone/dial");
     m->addParam("message","call.execute");
     m->addParam("pbxstate",m_state);
+    copyParams(*m,msg);
     Engine::enqueue(m);
     return true;
 }
@@ -640,11 +684,12 @@ bool PBXAssist::operTransfer(Message& msg)
     m->addParam("caller",m_keep.getValue("called"));
     m->addParam("called",msg.getValue("target"));
     m->addParam("pbxstate",m_state);
+    copyParams(*m,msg,&m_keep);
     // no error check as handling preroute is optional
     Engine::dispatch(m);
     *m = "call.route";
     if (!Engine::dispatch(m) || m->retValue().null() || (m->retValue() == "-") || (m->retValue() == "error")) {
-	delete m;
+	TelEngine::destruct(m);
 	return errorBeep();
     }
     m_state = "dial";
@@ -697,11 +742,12 @@ bool PBXAssist::operForTransfer(Message& msg)
     m->copyParam(m_keep,"caller");
     m->addParam("called",msg.getValue("target"));
     m->addParam("pbxstate",m_state);
+    copyParams(*m,msg,&m_keep);
     // no error check as handling preroute is optional
     Engine::dispatch(m);
     *m = "call.route";
     if (!Engine::dispatch(m) || m->retValue().null() || (m->retValue() == "-") || (m->retValue() == "error")) {
-	delete m;
+	TelEngine::destruct(m);
 	return errorBeep();
     }
     m_state = "fortransfer";
@@ -732,6 +778,7 @@ void PBXAssist::msgStartup(Message& msg)
 {
     DDebug(list(),DebugNote,"Copying startup parameters for '%s'",id().c_str());
     setGuest(msg);
+    setParams(msg);
     m_keep.setParam("billid",msg.getValue("billid"));
     NamedString* status = msg.getParam("status");
     if (status && (*status == "outgoing")) {
@@ -751,6 +798,7 @@ void PBXAssist::msgExecute(Message& msg)
     DDebug(list(),DebugNote,"Copying execute parameters for '%s'",id().c_str());
     // this gets only called on incoming call legs
     setGuest(msg);
+    setParams(msg);
     m_keep.setParam("billid",msg.getValue("billid"));
     m_keep.setParam("called",msg.getValue("called"));
     m_keep.setParam("caller",msg.getValue("caller"));
