@@ -40,7 +40,9 @@ public:
     ~WaveSource();
     virtual void run();
     virtual void cleanup();
+    virtual bool zeroRefsTest();
     void setNotify(const String& id);
+    bool derefReady();
 private:
     WaveSource(const char* file, CallEndpoint* chan, bool autoclose);
     void init(const String& file, bool autorepeat);
@@ -48,7 +50,7 @@ private:
     void detectWavFormat();
     void detectIlbcFormat();
     bool computeDataRate();
-    bool notify(DataSource* source, const char* reason = 0);
+    bool notify(WaveSource* source, const char* reason = 0);
     CallEndpoint* m_chan;
     DataBlock m_data;
     int m_fd;
@@ -61,6 +63,8 @@ private:
     bool m_autoclose;
     bool m_autoclean;
     bool m_nodata;
+    bool m_insert;
+    volatile bool m_derefOk;
 };
 
 class WaveConsumer : public DataConsumer
@@ -101,14 +105,14 @@ public:
 class Disconnector : public Thread
 {
 public:
-    Disconnector(CallEndpoint* chan, const String& id, DataSource* source, bool disc, const char* reason = 0);
+    Disconnector(CallEndpoint* chan, const String& id, WaveSource* source, bool disc, const char* reason = 0);
     virtual ~Disconnector();
     virtual void run();
     bool init();
 private:
     RefPointer<CallEndpoint> m_chan;
     Message* m_msg;
-    DataSource* m_source;
+    WaveSource* m_source;
     bool m_disc;
 };
 
@@ -137,6 +141,14 @@ private:
 };
 
 bool s_asyncDelete = true;
+bool s_dataPadding = true;
+bool s_pubReadable = false;
+
+#if defined (S_IRGRP) && defined(S_IROTH)
+#define CREATE_MODE (S_IRUSR|S_IWUSR|(s_pubReadable?(S_IRGRP|S_IROTH):0))
+#else
+#define CREATE_MODE (S_IRUSR|S_IWUSR)
+#endif
 
 INIT_PLUGIN(WaveFileDriver);
 
@@ -182,6 +194,12 @@ void WaveSource::init(const String& file, bool autorepeat)
 	m_format = "alaw";
     else if (file.endsWith(".mulaw") || file.endsWith(".u"))
 	m_format = "mulaw";
+    else if (file.endsWith(".2slin"))
+	m_format = "2*slin";
+    else if (file.endsWith(".2alaw"))
+	m_format = "2*alaw";
+    else if (file.endsWith(".2mulaw"))
+	m_format = "2*mulaw";
     else if (file.endsWith(".ilbc20"))
 	m_format = "ilbc20";
     else if (file.endsWith(".ilbc30"))
@@ -193,7 +211,7 @@ void WaveSource::init(const String& file, bool autorepeat)
     else if (file.endsWith(".lbc"))
 	detectIlbcFormat();
     else if (!file.endsWith(".slin"))
-	Debug(DebugMild,"Unknown format for file '%s', assuming signed linear",file.c_str());
+	Debug(DebugMild,"Unknown format for playback file '%s', assuming signed linear",file.c_str());
     if (computeDataRate()) {
 	if (autorepeat)
 	    m_repeatPos = ::lseek(m_fd,0,SEEK_CUR);
@@ -209,9 +227,11 @@ void WaveSource::init(const String& file, bool autorepeat)
 WaveSource::WaveSource(const char* file, CallEndpoint* chan, bool autoclose)
     : m_chan(chan), m_fd(-1), m_swap(false), m_brate(0), m_repeatPos(-1),
       m_total(0), m_time(0), m_autoclose(autoclose), m_autoclean(false),
-      m_nodata(false)
+      m_nodata(false), m_insert(false), m_derefOk(true)
 {
     Debug(&__plugin,DebugAll,"WaveSource::WaveSource(\"%s\",%p) [%p]",file,chan,this);
+    if (m_chan)
+	m_insert = true;
 }
 
 WaveSource::~WaveSource()
@@ -260,8 +280,10 @@ void WaveSource::detectAuFormat()
     }
     if (samp != 8000)
 	m_format << "/" << samp;
-    if (chan > 1)
+    if (chan > 1) {
 	m_format = String(chan) + "*" + m_format;
+	m_brate *= chan;
+    }
 }
 
 void WaveSource::detectWavFormat()
@@ -336,8 +358,17 @@ void WaveSource::run()
 	    }
 	    break;
 	}
-	if (r < (int)m_data.length())
-	    m_data.assign(m_data.data(),r);
+	if (r < (int)m_data.length()) {
+	    // if desired and possible extend last byte to fill buffer
+	    if (s_dataPadding && ((m_format == "mulaw") || (m_format == "alaw"))) {
+		unsigned char* d = (unsigned char*)m_data.data();
+		unsigned char last = d[r-1];
+		while (r < (int)m_data.length())
+		    d[r++] = last;
+	    }
+	    else
+		m_data.assign(m_data.data(),r);
+	}
 	if (m_swap) {
 	    uint16_t* p = (uint16_t*)m_data.data();
 	    for (int i = 0; i < r; i+= 2) {
@@ -351,9 +382,6 @@ void WaveSource::run()
 	    Thread::usleep((unsigned long)dly);
 	}
 	if (!alive()) {
-	    m_autoclose = false;
-	    // if this is a zombie it surely has no owner anymore
-	    m_chan = 0;
 	    notify(0,"replaced");
 	    return;
 	}
@@ -363,23 +391,53 @@ void WaveSource::run()
 	tpos += (r*(u_int64_t)1000000/m_brate);
     } while (r > 0);
     Debug(&__plugin,DebugAll,"WaveSource '%s' end of data (%u played) chan=%p [%p]",m_id.c_str(),m_total,m_chan,this);
+    if (!ref()) {
+	notify(0,"replaced");
+	return;
+    }
+    // prevent disconnector thread from succeeding before notify returns
+    m_derefOk = false;
     // at cleanup time deref the data source if we start no disconnector thread
     m_autoclean = !notify(this,"eof");
+    if (!deref())
+	m_derefOk = m_autoclean;
 }
 
 void WaveSource::cleanup()
 {
-    Debug(&__plugin,DebugAll,"WaveSource cleanup, total=%u, alive=%s, autoclean=%s [%p]",
-	m_total,String::boolText(alive()),String::boolText(m_autoclean),this);
+    Lock lock(DataEndpoint::commonMutex());
+    Debug(&__plugin,DebugAll,"WaveSource cleanup, total=%u, alive=%s, autoclean=%s chan=%p [%p]",
+	m_total,String::boolText(alive()),String::boolText(m_autoclean),m_chan,this);
+    clearThread();
     if (m_autoclean) {
 	asyncDelete(false);
-	if (m_chan && (m_chan->getSource() == this))
-	    m_chan->setSource();
+	if (m_insert) {
+	    if (m_chan && (m_chan->getSource() == this))
+		m_chan->setSource();
+	}
 	else
 	    deref();
 	return;
     }
-    ThreadedSource::cleanup();
+    if (m_derefOk)
+	ThreadedSource::cleanup();
+    else
+	m_derefOk = true;
+}
+
+bool WaveSource::zeroRefsTest()
+{
+    DDebug(&__plugin,DebugAll,"WaveSource::zeroRefsTest() chan=%p%s%s%s [%p]",
+	m_chan,
+	(thread() ? " thread" : ""),
+	(m_autoclose ? " close" : ""),
+	(m_autoclean ? " clean" : ""),
+	this);
+    // since this is a zombie it has no owner anymore and needs no removal
+    m_chan = 0;
+    m_autoclose = false;
+    m_autoclean = false;
+    return ThreadedSource::zeroRefsTest();
 }
 
 void WaveSource::setNotify(const String& id)
@@ -389,7 +447,19 @@ void WaveSource::setNotify(const String& id)
 	notify(this);
 }
 
-bool WaveSource::notify(DataSource* source, const char* reason)
+bool WaveSource::derefReady()
+{
+    for (int i = 0; i < 10; i++) {
+	if (m_derefOk)
+	    return true;
+	Thread::yield();
+    }
+    Debug(&__plugin,DebugWarn,"Source not deref ready, waiting more... [%p]",this);
+    Thread::msleep(10);
+    return m_derefOk;
+}
+
+bool WaveSource::notify(WaveSource* source, const char* reason)
 {
     if (!m_chan) {
 	if (m_id) {
@@ -403,8 +473,8 @@ bool WaveSource::notify(DataSource* source, const char* reason)
 	return false;
     }
     if (m_id || m_autoclose) {
-	DDebug(&__plugin,DebugInfo,"Preparing '%s' disconnector for '%s' chan '%s' source=%p [%p]",
-	    reason,m_id.c_str(),(m_chan ? m_chan->id().c_str() : ""),source,this);
+	DDebug(&__plugin,DebugInfo,"Preparing '%s' disconnector for '%s' chan %p '%s' source=%p [%p]",
+	    reason,m_id.c_str(),m_chan,(m_chan ? m_chan->id().c_str() : ""),source,this);
 	Disconnector *disc = new Disconnector(m_chan,m_id,source,m_autoclose,reason);
 	return disc->init();
     }
@@ -430,6 +500,12 @@ WaveConsumer::WaveConsumer(const String& file, CallEndpoint* chan, unsigned maxl
 	m_format = "alaw";
     else if (file.endsWith(".mulaw") || file.endsWith(".u"))
 	m_format = "mulaw";
+    else if (file.endsWith(".2slin"))
+	m_format = "2*slin";
+    else if (file.endsWith(".2alaw"))
+	m_format = "2*alaw";
+    else if (file.endsWith(".2mulaw"))
+	m_format = "2*mulaw";
     else if (file.endsWith(".ilbc20"))
 	m_format = "ilbc20";
     else if (file.endsWith(".ilbc30"))
@@ -438,7 +514,9 @@ WaveConsumer::WaveConsumer(const String& file, CallEndpoint* chan, unsigned maxl
 	m_header=Ilbc;
     else if (file.endsWith(".au"))
 	m_header=Au;
-    m_fd = ::open(file.safe(),O_WRONLY|O_CREAT|O_TRUNC|O_NOCTTY|O_BINARY,S_IRUSR|S_IWUSR);
+    else if (!file.endsWith(".slin"))
+	Debug(DebugMild,"Unknown format for recorded file '%s', assuming signed linear",file.c_str());
+    m_fd = ::open(file.safe(),O_WRONLY|O_CREAT|O_TRUNC|O_NOCTTY|O_BINARY,CREATE_MODE);
     if (m_fd < 0)
 	Debug(DebugWarn,"Creating '%s': error %d: %s",
 	    file.c_str(), errno, ::strerror(errno));
@@ -567,8 +645,8 @@ void WaveConsumer::Consume(const DataBlock& data, unsigned long tStamp)
 		m_fd = -1;
 	    }
 	    if (m_chan) {
-		DDebug(&__plugin,DebugInfo,"Preparing 'maxlen' disconnector for '%s' chan '%s' in consumer [%p]",
-		    m_id.c_str(),(m_chan ? m_chan->id().c_str() : ""),this);
+		DDebug(&__plugin,DebugInfo,"Preparing 'maxlen' disconnector for '%s' chan %p '%s' in consumer [%p]",
+		    m_id.c_str(),m_chan,(m_chan ? m_chan->id().c_str() : ""),this);
 		Disconnector *disc = new Disconnector(m_chan,m_id,0,false,"maxlen");
 		m_chan = 0;
 		disc->init();
@@ -578,19 +656,29 @@ void WaveConsumer::Consume(const DataBlock& data, unsigned long tStamp)
 }
 
 
-Disconnector::Disconnector(CallEndpoint* chan, const String& id, DataSource* source, bool disc, const char* reason)
-    : m_chan(chan), m_msg(0), m_source(source), m_disc(disc)
+Disconnector::Disconnector(CallEndpoint* chan, const String& id, WaveSource* source, bool disc, const char* reason)
+    : Thread("WaveDisconnector"),
+      m_chan(chan), m_msg(0), m_source(0), m_disc(disc)
 {
     if (id) {
 	Message* m = new Message("chan.notify");
-	if (chan)
-	    m->addParam("id",chan->id());
+	if (m_chan)
+	    m->addParam("id",m_chan->id());
 	m->addParam("targetid",id);
 	if (reason)
 	    m->addParam("reason",reason);
-	m->userData(chan);
+	m->userData(m_chan);
 	m_msg = m;
     }
+    if (source) {
+	if (source->ref())
+	    m_source = source;
+	else {
+	    Debug(&__plugin,DebugGoOn,"Disconnecting dead source %p, reason: '%s'",
+		source,reason);
+	    m_chan = 0;
+	}
+    }
 }
 
 Disconnector::~Disconnector()
@@ -604,7 +692,7 @@ Disconnector::~Disconnector()
 bool Disconnector::init()
 {
     if (error()) {
-	Debug(DebugFail,"Error creating disconnector thread %p",this);
+	Debug(&__plugin,DebugGoOn,"Error creating disconnector thread %p",this);
 	delete this;
 	return false;
     }
@@ -613,8 +701,8 @@ bool Disconnector::init()
 
 void Disconnector::run()
 {
-    DDebug(&__plugin,DebugAll,"Disconnector::run() chan=%p msg=%p source=%s disc=%s [%p]",
-	(void*)m_chan,m_msg,String::boolText(m_source),String::boolText(m_disc),this);
+    DDebug(&__plugin,DebugAll,"Disconnector::run() chan=%p msg=%p source=%p disc=%s [%p]",
+	(void*)m_chan,m_msg,m_source,String::boolText(m_disc),this);
     if (!m_chan)
 	return;
     if (m_source) {
@@ -623,6 +711,9 @@ void Disconnector::run()
 	else
 	    Debug(&__plugin,DebugMild,"Source %p in channel %p was replaced with %p",
 		m_source,(void*)m_chan,m_chan->getSource());
+	if (!m_source->derefReady())
+	    Debug(&__plugin,DebugGoOn,"Source %p is not deref ready, crash may occur",m_source);
+	m_source->deref();
 	if (m_disc)
 	    m_chan->disconnect("eof");
     }
@@ -977,6 +1068,8 @@ void WaveFileDriver::initialize()
     Output("Initializing module WaveFile");
     setup();
     s_asyncDelete = Engine::config().getBoolValue("hacks","asyncdelete",true);
+    s_dataPadding = Engine::config().getBoolValue("hacks","datapadding",true);
+    s_pubReadable = Engine::config().getBoolValue("hacks","wavepubread",false);
     if (!m_handler) {
 	m_handler = new AttachHandler;
 	Engine::install(m_handler);
