@@ -62,9 +62,14 @@ extern "C" {
 #define WP_ERR_CRC   0x02
 #define WP_ERR_ABORT 0x04
 
+#define WP_RPT_REPEAT 0                  // Repeat flag in header
+#define WP_RPT_LEN 1                     // Repeated data length
+#define WP_RPT_DATA 2                    // Repeated data offset in header
+#define WP_RPT_MAXDATA 8                 // Max repeated data length
+
 #define MAX_PACKET 1200
 
-#define MAX_READ_ERRORS 250              // WpData::run(): Display read error message
+#define MAX_READ_ERRORS 250              // WpSpan::run(): Display read error message
 #define WPSOCKET_SELECT_TIMEOUT 125      // Value used in WpSocket::select() to timeout
 
 using namespace TelEngine;
@@ -77,9 +82,9 @@ class WpSigThread;                       // D-channel read data
 class WpSource;                          // Data source
 class WpConsumer;                        // Data consumer
 class WpCircuit;                         // Single Wanpipe B-channel (SignallingCircuit)
-class WpData;                            // Wanpipe span B-channel group
-class WpDataThread;                      // B-channel group read/write data
-class WpConfig;                          // Configuration file
+class WpSpan;                            // Wanpipe span B-channel group
+class WpSpanThread;                      // B-channel group read/write data
+class WpModule;                          // The driver
 
 // Implements a circular queue for data consumer
 class Fifo
@@ -110,7 +115,7 @@ private:
     unsigned int m_tail;
 };
 
-// I/O socket for WpInterface and WpData
+// I/O socket for WpInterface and WpSpan
 class WpSocket
 {
 public:
@@ -169,12 +174,15 @@ class WpInterface : public SignallingInterface
 {
     friend class WpSigThread;
 public:
-    // Create an instance of WpInterface or WpData
+    // Create an instance of WpInterface or WpSpan
     static void* create(const String& type, const NamedList& name);
     WpInterface(const NamedList& params);
     virtual ~WpInterface();
     // Initialize interface. Return false on failure
-    bool init(NamedList& params);
+    bool init(const NamedList& config, NamedList& params);
+    // Remove links. Dispose memory
+    virtual void destruct()
+	{ cleanup(true); }
     // Send signalling packet
     virtual bool transmitPacket(const DataBlock& packet, bool repeat, PacketType type);
     // Interface control
@@ -184,13 +192,20 @@ protected:
     // Read data from socket
     bool receiveAttempt();
 private:
+    inline void cleanup(bool release) {
+	    control(Disable,0);
+	    attach(0);
+	    if (release)
+		GenObject::destruct();
+	}
     WpSocket m_socket;
     WpSigThread* m_thread;               // Thread used to read data from socket
     bool m_readOnly;                     // Readonly interface
-    bool m_received;                     // Received data flag
+    int m_notify;                        // Upper layer notification on received data (0: success. 1: not notified. 2: notified)
     int m_overRead;                      // Header extension
     unsigned char m_errorMask;           // Error mask to filter received errors
     bool m_sendReadOnly;                 // Print send attempt on readonly interface error
+    SignallingTimer m_timerRxUnder;      // RX underrun notification
 };
 
 // Read signalling data for WpInterface
@@ -199,7 +214,7 @@ class WpSigThread : public Thread
     friend class WpInterface;
 public:
     inline WpSigThread(WpInterface* iface, Priority prio = Normal)
-	: Thread("WpSignal",prio), m_interface(iface)
+	: Thread("WpInterfaceThread",prio), m_interface(iface)
 	{}
     virtual ~WpSigThread();
     virtual void run();
@@ -246,7 +261,7 @@ protected:
 class WpCircuit : public SignallingCircuit
 {
 public:
-    WpCircuit(unsigned int code, SignallingCircuitGroup* group, WpData* data,
+    WpCircuit(unsigned int code, SignallingCircuitGroup* group, WpSpan* data,
 	unsigned int buflen);
     virtual ~WpCircuit();
     virtual bool status(Status newStat, bool sync = false);
@@ -265,14 +280,14 @@ private:
 };
 
 // Wanpipe B-channel group
-class WpData : public SignallingCircuitSpan
+class WpSpan : public SignallingCircuitSpan
 {
-    friend class WpDataThread;
+    friend class WpSpanThread;
 public:
-    WpData(const NamedList& params);
-    virtual ~WpData();
+    WpSpan(const NamedList& params);
+    virtual ~WpSpan();
     // Initialize data channel span. Return false on failure
-    bool init(NamedList& params);
+    bool init(const NamedList& config, const NamedList& defaults, NamedList& params);
     // Swap data if necessary
     inline unsigned char swap(unsigned char c)
 	{ return m_swap ? s_bitswap[c] : c; }
@@ -282,10 +297,6 @@ public:
     // Sent data from each circuit is merged into one data block
     void run();
 protected:
-    // Process a list of circuits to create. Update m_count
-    // Validate each circuit number to be positive and lesser or equal to m_chans
-    // Return the codes of the circuits or 0 on error
-    unsigned int* processCicList(const String& cicList);
     // Create circuits (all or nothing)
     // delta: number to add to each circuit code
     // cicList: Circuits to create
@@ -301,7 +312,7 @@ protected:
     static unsigned char s_bitswap[256];
 private:
     WpSocket m_socket;
-    WpDataThread* m_thread;
+    WpSpanThread* m_thread;
     bool m_canSend;                      // Can send data (not a readonly span)
     bool m_swap;                         // Swap bits flag
     unsigned int m_chans;                // Total number of circuits for this span
@@ -318,28 +329,33 @@ private:
 };
 
 // B-channel group read/write data
-class WpDataThread : public Thread
+class WpSpanThread : public Thread
 {
-    friend class WpData;
+    friend class WpSpan;
 public:
-    inline WpDataThread(WpData* data, Priority prio = Normal)
-	: Thread("WpData",prio), m_data(data)
+    inline WpSpanThread(WpSpan* data, Priority prio = Normal)
+	: Thread("WpSpanThread",prio), m_data(data)
 	{}
-    virtual ~WpDataThread();
+    virtual ~WpSpanThread();
     virtual void run();
 private:
-    WpData* m_data;
+    WpSpan* m_data;
 };
 
-class WpConfig : public Configuration
+// The driver
+class WpModule : public DebugEnabler
 {
 public:
-    inline WpConfig() : Configuration(Engine::configFile("wpcard"))
-	{ load(); }
+    WpModule();
+    ~WpModule();
 };
 
+
+static WpModule driver;
 YSIGFACTORY2(WpInterface,SignallingInterface);
 
+static Mutex s_ifaceNotify(true);        // WpInterface: lock recv data notification counter
+
 /**
  * Fifo
  */
@@ -482,23 +498,40 @@ bool WpSocket::select(unsigned int multiplier)
 /**
  * WpInterface
  */
-// Create WpInterface or WpData
+// Create WpInterface or WpSpan
 void* WpInterface::create(const String& type, const NamedList& name)
 {
-    if (type == "sig") {
+    bool interface = false;
+    if (type == "sig")
+	interface = true;
+    else  if (type == "voice")
+	;
+    else
+	return 0;
+
+    Configuration cfg(Engine::configFile("wpcard"));
+    cfg.load();
+    const char* sectName = name.getValue(type);
+    DDebug(&driver,DebugAll,"Factory trying to create %s='%s'",type.c_str(),sectName);
+    NamedList* config = cfg.getSection(sectName);
+    if (!config) {
+	DDebug(&driver,DebugAll,"No section '%s' in configuration",c_safe(sectName));
+	return 0;
+    }
+
+    if (interface) {
 	WpInterface* iface = new WpInterface(name);
-	if (iface->init((NamedList&)name))
+	if (iface->init(*config,(NamedList&)name))
 	    return iface;
-	delete iface;
-	return 0;
-    }
-    if (type == "voice") {
-	WpData* data = new WpData(name);
-	if (data->init((NamedList&)name))
-	    return data;
-	delete data;
+	TelEngine::destruct(iface);
 	return 0;
     }
+    NamedList* general = cfg.getSection("general");
+    NamedList dummy("general");
+    WpSpan* data = new WpSpan(name);
+    if (data->init(*config,general?*general:dummy,(NamedList&)name))
+	return data;
+    TelEngine::destruct(data);
     return 0;
 }
 
@@ -506,9 +539,10 @@ WpInterface::WpInterface(const NamedList& params)
     : m_socket(this),
     m_thread(0),
     m_readOnly(false),
-    m_received(false),
+    m_notify(0),
     m_overRead(0),
-    m_sendReadOnly(false)
+    m_sendReadOnly(false),
+    m_timerRxUnder(0)
 {
     setName(params.getValue("debugname","WpInterface"));
     XDebug(this,DebugAll,"WpInterface::WpInterface() [%p]",this);
@@ -516,39 +550,39 @@ WpInterface::WpInterface(const NamedList& params)
 
 WpInterface::~WpInterface()
 {
-    control(Disable,0);
+    cleanup(false);
     XDebug(this,DebugAll,"WpInterface::~WpInterface() [%p]",this);
 }
 
-bool WpInterface::init(NamedList& params)
+bool WpInterface::init(const NamedList& config, NamedList& params)
 {
-    WpConfig cfg;
-    const char* sectName = params.getValue("sig");
-    NamedList* sect = cfg.getSection(sectName);
-    if (!sect) {
-	Debug(this,DebugNote,"Missing section '%s' in configuration",c_safe(sectName));
-	return false;
-    }
     // Set socket card / device
-    m_socket.card(sectName);
-    const char* sig = params.getValue("siggroup",sect->getValue("siggroup"));
+    m_socket.card(config);
+    const char* sig = params.getValue("siggroup",config.getValue("siggroup"));
     if (!sig) {
-	Debug(this,DebugNote,"Missing or invalid signalling group '%s' in configuration",sig);
+	Debug(this,DebugWarn,
+	    "Missing or invalid siggroup='%s' in configuration [%p]",
+	    c_safe(sig),this);
 	return false;
     }
     m_socket.device(sig);
 
-    m_readOnly = sect->getBoolValue("readonly",false);
+    m_readOnly = config.getBoolValue("readonly",false);
 
-    int i = params.getIntValue("errormask",sect->getIntValue("errormask",255));
+    int i = params.getIntValue("errormask",config.getIntValue("errormask",255));
     m_errorMask = ((i >= 0 && i < 256) ? i : 255);
 
+    int rx = params.getIntValue("rxunderruninterval");
+    if (rx > 0)
+	m_timerRxUnder.interval(rx);
+
     if (debugAt(DebugInfo)) {
 	String s;
-	s << "\r\nCard:       " << m_socket.card();
-	s << "\r\nDevice:     " << m_socket.device();
-	s << "\r\nError mask: " << (unsigned int)m_errorMask;
-	s << "\r\nRead only:  " << String::boolText(m_readOnly);
+	s << "\r\nCard:                  " << m_socket.card();
+	s << "\r\nDevice:                " << m_socket.device();
+	s << "\r\nError mask:            " << (unsigned int)m_errorMask;
+	s << "\r\nRead only:             " << String::boolText(m_readOnly);
+	s << "\r\nRX underrun interval:  " << (unsigned int)m_timerRxUnder.interval() << " ms";
 	Debug(this,DebugInfo,"Initialized: [%p]%s",this,s.c_str());
     }
     return true;
@@ -575,24 +609,43 @@ bool WpInterface::transmitPacket(const DataBlock& packet, bool repeat, PacketTyp
     }
 #endif
 
+#ifdef wp_api_tx_hdr_hdlc_rpt_data
+    // Repeat supported and data not too big
+    if (repeat)
+	if (packet.length() <= WP_RPT_MAXDATA) {
+	    unsigned char hdr[WP_HEADER];
+	    ::memset(hdr,0,WP_HEADER);
+	    hdr[WP_RPT_REPEAT] = 1;
+	    hdr[WP_RPT_LEN] = packet.length();
+	    ::memcpy(hdr+WP_RPT_DATA,packet.data(),packet.length());
+	    return -1 != m_socket.send(hdr,WP_HEADER,0);
+	}
+	else
+	    Debug(this,DebugWarn,"Can't repeat packet (type=%u) with length=%u",
+		type,packet.length());
+#endif
+
     DataBlock data(0,WP_HEADER);
     data += packet;
-    unsigned char* d = static_cast<unsigned char*>(data.data());
-    if (repeat)
-	d[WP_WR_FORCE] = 1;
-    switch (type) {
-	case SS7Fisu:
-	    d[WP_WR_TYPE] = WANOPT_SS7_FISU;
-	    break;
-	case SS7Lssu:
-	    d[WP_WR_TYPE] = WANOPT_SS7_LSSU;
-	    break;
-	default:
-	    break;
-    }
+
     return -1 != m_socket.send(data.data(),data.length(),0);
 }
 
+static inline const char* error(unsigned char err)
+{
+    static String s;
+    s.clear();
+    if (err & WP_ERR_CRC)
+	s.append("CRC");
+    if (err & WP_ERR_FIFO)
+	s.append("RxOver"," ");
+    if (err & WP_ERR_ABORT)
+	s.append("Align"," ");
+    if (s.null())
+	s << (int)err;
+    return s.safe();
+}
+
 // Receive signalling packet
 bool WpInterface::receiveAttempt()
 {
@@ -612,8 +665,8 @@ bool WpInterface::receiveAttempt()
 	r -= (WP_HEADER + m_overRead);
 	unsigned char err = buf[WP_RD_ERROR] & m_errorMask;
 	if (err) {
-	    DDebug(this,DebugWarn,"Packet got error: %u [%p]",
-		buf[WP_RD_ERROR],this);
+	    DDebug(this,DebugWarn,"Packet got error: %u (%s) [%p]",
+		buf[WP_RD_ERROR],error(buf[WP_RD_ERROR]),this);
 	    if (err & WP_ERR_FIFO)
 		notify(RxOverflow);
 	    if (err & WP_ERR_CRC)
@@ -623,15 +676,18 @@ bool WpInterface::receiveAttempt()
 	    return true;
 	}
 
+	s_ifaceNotify.lock();
+	m_notify = 0;
+	s_ifaceNotify.unlock();
+
 #ifdef XDEBUG
-    if (debugAt(DebugAll)) {
-	String str;
-	str.hexify(buf+WP_HEADER,r,' ');
-	Debug(this,DebugAll,"Received %d bytes: %s",r,str.c_str());
-    }
+	if (debugAt(DebugAll)) {
+	    String str;
+	    str.hexify(buf+WP_HEADER,r,' ');
+	    Debug(this,DebugAll,"Received %d bytes: %s",r,str.c_str());
+	}
 #endif
 
-	m_received = true;
 	DataBlock data(buf+WP_HEADER,r);
 	receivedPacket(data);
 	data.clear(false);
@@ -644,10 +700,19 @@ bool WpInterface::receiveAttempt()
 // Disable: Cancel thread. Close socket
 bool WpInterface::control(Operation oper, NamedList* params)
 {
+    DDebug(this,DebugAll,"Control with oper=%u [%p]",oper,this);
     switch (oper) {
 	case Enable:
 	case Disable:
 	    break;
+	case EnableTx:
+	case DisableTx:
+	    if (m_readOnly == (oper == DisableTx))
+		return true;
+	    m_readOnly = (oper == DisableTx);
+	    m_sendReadOnly = false;
+	    Debug(this,DebugInfo,"Tx is %sabled [%p]",m_readOnly?"dis":"en",this);
+	    return true;
 	case Query:
 	    return m_socket.valid() && m_thread && m_thread->running();
 	default:
@@ -663,8 +728,10 @@ bool WpInterface::control(Operation oper, NamedList* params)
 	    else
 		ok = m_thread->startup();
 	}
-	if (ok)
+	if (ok) {
 	    DDebug(this,DebugAll,"Enabled [%p]",this);
+	    m_timerRxUnder.start();
+	}
 	else {
 	    Debug(this,DebugWarn,"Enable failed [%p]",this);
 	    control(Disable,0);
@@ -672,6 +739,7 @@ bool WpInterface::control(Operation oper, NamedList* params)
 	return ok;
     }
     // oper is Disable
+    m_timerRxUnder.stop();
     if (m_thread) {
 	m_thread->cancel();
 	while (m_thread)
@@ -684,11 +752,21 @@ bool WpInterface::control(Operation oper, NamedList* params)
 
 void WpInterface::timerTick(const Time& when)
 {
-    if (m_received)
-	m_received = false;
-    else {
-//	XDebug(this,DebugAll,"Not received any packets in the last tick [%p]",this);
+    if (!m_timerRxUnder.timeout(when.msec()))
+	return;
+    s_ifaceNotify.lock();
+    if (m_notify) {
+	if (m_notify == 1) {
+	    DDebug(this,DebugMild,"RX idle for " FMT64 "ms. Notifying receiver [%p]",
+		m_timerRxUnder.interval(),this);
+	    notify(RxUnderrun);
+	    m_notify = 2;
+	}
     }
+    else
+	m_notify = 1;
+    s_ifaceNotify.unlock();
+    m_timerRxUnder.start(when.msec());
 }
 
 /**
@@ -703,7 +781,7 @@ WpSigThread::~WpSigThread()
 
 void WpSigThread::run()
 {
-    DDebug(m_interface,DebugAll,"WpSigThread::run() [%p]",this);
+    DDebug(m_interface,DebugAll,"%s start running [%p]",name(),this);
     for (;;) {
 	Thread::yield(true);
 	while (m_interface && m_interface->receiveAttempt())
@@ -776,7 +854,7 @@ void WpConsumer::Consume(const DataBlock& data, unsigned long tStamp)
 /**
  * WpCircuit
  */
-WpCircuit::WpCircuit(unsigned int code, SignallingCircuitGroup* group, WpData* data,
+WpCircuit::WpCircuit(unsigned int code, SignallingCircuitGroup* group, WpSpan* data,
 	unsigned int buflen)
     : SignallingCircuit(TDM,code,Idle,group,data),
     m_mutex(true),
@@ -788,12 +866,12 @@ WpCircuit::WpCircuit(unsigned int code, SignallingCircuitGroup* group, WpData* d
     if (buflen) {
 	m_source = new WpSource(this,"alaw",buflen);
 	m_consumer = new WpConsumer(this,"alaw",buflen);
-	XDebug(group,DebugAll,"WpCircuit %u. Source (%p). Consumer (%p) [%p]",
+	XDebug(group,DebugAll,"WpCircuit(%u). Source (%p). Consumer (%p) [%p]",
 	    code,m_source,m_consumer,this);
     }
     else
 	Debug(group,DebugNote,
-	    "WpCircuit %u. No source and consumer. Buffer length is 0 [%p]",
+	    "WpCircuit(%u). No source and consumer. Buffer length is 0 [%p]",
 	    code,this);
 }
 
@@ -826,16 +904,17 @@ bool WpCircuit::status(Status newStat, bool sync)
 	    break;
 	default: ;
 	    Debug(group(),DebugNote,
-		"WpCircuit %u. Can't change status to unhandled value %u [%p]",
+		"WpCircuit(%u). Can't change status to unhandled value %u [%p]",
 		code(),newStat,this);
 	    return false;
     }
     if (SignallingCircuit::status() == Missing) {
 	Debug(group(),DebugNote,
-	    "WpCircuit %u. Can't change status to '%u'. Circuit is missing [%p]",
+	    "WpCircuit(%u). Can't change status to '%u'. Circuit is missing [%p]",
 	    code(),newStat,this);
 	return false;
     }
+    Status oldStat = SignallingCircuit::status();
     // Change status
     if (!SignallingCircuit::status(newStat,sync))
 	return false;
@@ -846,9 +925,8 @@ bool WpCircuit::status(Status newStat, bool sync)
 	enableData = true;
     // Don't put this message for final states
     if (!Engine::exiting())
-	DDebug(group(),DebugAll,
-	    "WpCircuit %u. Changed status to '%u'. %s data transfer [%p]",
-	    code(),newStat,enableData ? "Enable" : "Disable",this);
+	DDebug(group(),DebugAll,"WpCircuit(%u). Changed status to %u [%p]",
+	    code(),newStat,this);
     if (enableData) {
 	m_sourceValid = m_source;
 	m_consumerValid = m_consumer;
@@ -856,24 +934,25 @@ bool WpCircuit::status(Status newStat, bool sync)
     }
     // Disable data if not already disabled
     if (m_consumerValid) {
-	m_consumerValid = 0;
-	XDebug(group(),DebugAll,"WpCircuit %u. Consumer transferred %u byte(s) [%p]",
-	    code(),m_consumer->m_total,this);
-	if (m_consumer->m_errorCount)
-	    DDebug(group(),DebugMild,
-	        "WpCircuit %u. Consumer errors: %u. Lost: %u/%u [%p]",
-		code(),m_consumer->m_errorCount,m_consumer->m_errorBytes,
-		m_consumer->m_total,this);
+	if (oldStat == Connected) {
+	    XDebug(group(),DebugAll,"WpCircuit(%u). Consumer transferred %u byte(s) [%p]",
+		code(),m_consumer->m_total,this);
+	    if (m_consumer->m_errorCount)
+		DDebug(group(),DebugMild,"WpCircuit(%u). Consumer errors: %u. Lost: %u/%u [%p]",
+		    code(),m_consumer->m_errorCount,m_consumer->m_errorBytes,
+		    m_consumer->m_total,this);
+	}
 	m_consumer->clear();
+	m_consumerValid = 0;
 	m_consumer->m_errorCount = m_consumer->m_errorBytes = 0;
 	m_consumer->m_total = 0;
     }
     if (m_sourceValid) {
-	// Remove consumer
+	if (oldStat == Connected)
+	    XDebug(group(),DebugAll,"WpCircuit(%u). Source transferred %u byte(s) [%p]",
+		code(),m_source->m_total,this);
 	m_source->clear();
 	m_sourceValid = 0;
-	XDebug(group(),DebugAll,"WpCircuit %u. Source transferred %u byte(s) [%p]",
-	    code(),m_source->m_total,this);
 	m_source->m_total = 0;
     }
     return true;
@@ -890,7 +969,7 @@ bool WpCircuit::updateFormat(const char* format, int direction)
     if (direction == -1 || direction == 0) {
 	if (m_consumer && m_consumer->getFormat() != format) {
 	    m_consumer->changeFormat(format);
-	    DDebug(group(),DebugAll,"WpCircuit %u. Consumer format set to '%s' [%p]",
+	    DDebug(group(),DebugAll,"WpCircuit(%u). Consumer format set to '%s' [%p]",
 		code(),format,this);
 	}
 	else
@@ -899,7 +978,7 @@ bool WpCircuit::updateFormat(const char* format, int direction)
     if (direction == 1 || direction == 0) {
 	if (m_source && m_source->getFormat() != format) {
 	    m_source->changeFormat(format);
-	    DDebug(group(),DebugAll,"WpCircuit %u. Source format set to '%s' [%p]",
+	    DDebug(group(),DebugAll,"WpCircuit(%u). Source format set to '%s' [%p]",
 		code(),format,this);
 	}
 	else
@@ -921,9 +1000,9 @@ void* WpCircuit::getObject(const String& name) const
 }
 
 /**
- * WpData
+ * WpSpan
  */
-unsigned char WpData::s_bitswap[256] = {
+unsigned char WpSpan::s_bitswap[256] = {
 	0x00,0x80,0x40,0xc0,0x20,0xa0,0x60,0xe0,0x10,0x90,0x50,0xd0,0x30,0xb0,0x70,0xf0,0x08,0x88,0x48,0xc8,
 	0x28,0xa8,0x68,0xe8,0x18,0x98,0x58,0xd8,0x38,0xb8,0x78,0xf8,0x04,0x84,0x44,0xc4,0x24,0xa4,0x64,0xe4,
 	0x14,0x94,0x54,0xd4,0x34,0xb4,0x74,0xf4,0x0c,0x8c,0x4c,0xcc,0x2c,0xac,0x6c,0xec,0x1c,0x9c,0x5c,0xdc,
@@ -941,7 +1020,7 @@ unsigned char WpData::s_bitswap[256] = {
 
 // Initialize B-channel group
 // Create circuits. Start worker thread
-WpData::WpData(const NamedList& params)
+WpSpan::WpSpan(const NamedList& params)
     : SignallingCircuitSpan(params.getValue("debugname"),
 	static_cast<SignallingCircuitGroup*>(params.getObject("SignallingCircuitGroup"))),
     m_socket(m_group),
@@ -959,12 +1038,12 @@ WpData::WpData(const NamedList& params)
     m_buffer(0),
     m_bufferLen(0)
 {
-    XDebug(m_group,DebugAll,"WpData::WpData(). Name '%s' [%p]",id().safe(),this);
+    XDebug(m_group,DebugAll,"WpSpan::WpSpan(). Name '%s' [%p]",id().safe(),this);
 }
 
 // Terminate worker thread
 // Close socket. Clear circuit list
-WpData::~WpData()
+WpSpan::~WpSpan()
 {
     if (m_thread) {
 	m_thread->cancel();
@@ -976,40 +1055,31 @@ WpData::~WpData()
 	delete[] m_circuits;
     if (m_buffer)
 	delete[] m_buffer;
-    XDebug(m_group,DebugAll,"WpData::~WpData() [%p]",this);
+    XDebug(m_group,DebugAll,"WpSpan::~WpSpan() [%p]",this);
 }
 
 // Initialize
-bool WpData::init(NamedList& params)
+bool WpSpan::init(const NamedList& config, const NamedList& defaults, NamedList& params)
 {
-    XDebug(m_group,DebugAll,"WpData('%s'). Initializing [%p]",id().safe(),this);
     if (!m_group) {
-	Debug(DebugNote,"WpData('%s'). Circuit group is missing [%p]",
+	Debug(DebugNote,"WpSpan('%s'). Circuit group is missing [%p]",
 	    id().safe(),this);
 	return false;
     }
-    WpConfig cfg;
-    const char* sectName = params.getValue("voice");
-    NamedList* sect = cfg.getSection(sectName);
-    if (!sect) {
-	Debug(m_group,DebugNote,"WpData('%s'). No section '%s' in configuration [%p]",
-	    id().safe(),c_safe(sectName),this);
-	return false;
-    }
     // Set socket card / device
-    m_socket.card(sectName);
-    const char* voice = params.getValue("voicegroup",sect->getValue("voicegroup"));
+    m_socket.card(config);
+    const char* voice = params.getValue("voicegroup",config.getValue("voicegroup"));
     if (!voice) {
-	Debug(m_group,DebugNote,"WpData('%s'). Missing or invalid voice group [%p]",
+	Debug(m_group,DebugNote,"WpSpan('%s'). Missing or invalid voice group [%p]",
 	    id().safe(),this);
 	return false;
     }
     m_socket.device(voice);
-    m_canSend = !sect->getBoolValue("readonly",false);
+    m_canSend = !config.getBoolValue("readonly",false);
     // Type depending data: channel count, samples, circuit list
-    String type = sect->getValue("type");
-    String cics = sect->getValue("voicechans");
-    m_samples = params.getIntValue("samples",sect->getIntValue("samples"));
+    String type = config.getValue("type");
+    String cics = config.getValue("voicechans");
+    m_samples = params.getIntValue("samples",config.getIntValue("samples"));
     if (type.null())
 	type = "E1";
     if (type == "E1") {
@@ -1027,32 +1097,32 @@ bool WpData::init(NamedList& params)
 	    m_samples = 64;
     }
     else {
-	Debug(m_group,DebugNote,"WpData('%s'). Invalid voice group type '%s' [%p]",
+	Debug(m_group,DebugNote,"WpSpan('%s'). Invalid voice group type '%s' [%p]",
 	    id().safe(),type.safe(),this);
 	return false;
     }
     params.setParam("chans",String(m_chans));
     // Other data
-    m_swap = cfg.getBoolValue("general","bitswap",true);
-    m_noData = cfg.getIntValue("general","idlevalue",0xff);
-    m_buflen = cfg.getIntValue("general","buflen",160);
-    m_swap = params.getBoolValue("bitswap",sect->getBoolValue("bitswap",m_swap));
-    m_noData = params.getIntValue("idlevalue",sect->getIntValue("idlevalue",m_noData));
-    m_buflen = params.getIntValue("buflen",sect->getIntValue("buflen",m_buflen));
+    m_swap = defaults.getBoolValue("bitswap",true);
+    m_noData = defaults.getIntValue("idlevalue",0xff);
+    m_buflen = defaults.getIntValue("buflen",160);
+    m_swap = params.getBoolValue("bitswap",config.getBoolValue("bitswap",m_swap));
+    m_noData = params.getIntValue("idlevalue",config.getIntValue("idlevalue",m_noData));
+    m_buflen = params.getIntValue("buflen",config.getIntValue("buflen",m_buflen));
     // Buffer length can't be 0
     if (!m_buflen)
 	m_buflen = 160;
     // Channels
     if (!createCircuits(params.getIntValue("start"),cics)) {
 	Debug(m_group,DebugNote,
-	    "WpData('%s'). Failed to create voice chans (voicechans=%s) [%p]",
+	    "WpSpan('%s'). Failed to create voice chans (voicechans=%s) [%p]",
 	    id().safe(),cics.safe(),this);
 	return false;
     }
     // Start processing data
-    m_thread = new WpDataThread(this);
+    m_thread = new WpSpanThread(this);
     if (!m_thread->startup()) {
-	Debug(m_group,DebugNote,"WpData('%s'). Failed to start worker thread [%p]",
+	Debug(m_group,DebugNote,"WpSpan('%s'). Failed to start worker thread [%p]",
 	    id().safe(),this);
 	return false;
     }
@@ -1068,76 +1138,18 @@ bool WpData::init(NamedList& params)
 	s << "\r\nBuffer length:  " << (unsigned int)m_buflen;
 	s << "\r\nUsed channels:  " << m_count;
 	s << "\r\nRead only:      " << String::boolText(!m_canSend);
-	Debug(m_group,DebugInfo,"WpData('%s'). Initialized: [%p]%s",
+	Debug(m_group,DebugInfo,"WpSpan('%s'). Initialized: [%p]%s",
 	    id().safe(),this,s.c_str());
     }
     return true;
 }
 
-// Process a list of circuits to create. Update m_count
-// Validate each circuit number to be positive and lesser or equal to m_chans
-// Return the codes of the circuits or 0 on error
-// List may be separated by '.' or ',' and may contain code intervals
-unsigned int* WpData::processCicList(const String& cicList)
-{
-    ObjList* listSplit = 0;
-    char separator = (-1 != cicList.find(',')) ? ',' : '.';
-    listSplit = cicList.split(separator,false);
-    if (!(listSplit || listSplit->count()))
-	return 0;
-    // Split the intervals into single code elements
-    unsigned int* cicCodes = new unsigned int[m_chans];
-    bool ok = true;
-    ObjList* o = listSplit->skipNull();
-    for (; o; o = o->skipNext()) {
-	String* s = static_cast<String*>(o->get());
-	if (s->null())
-	    continue;
-	// Check for interval
-	int sep = s->find('-');
-	int first = -1, last = -2;
-	if (sep == -1)
-	    first = last = s->toInteger(-1);
-	else {
-	    String tmp = s->substr(0,sep);
-	    first = tmp.toInteger(-1);
-	    tmp = s->substr(sep + 1);
-	    last = tmp.toInteger(-2);
-	}
-	// Check for valid interval and codes
-	// The interval must be [1..m_chans]
-	if (first <= 0 || last <= 0 || last < first || (unsigned int)last > m_chans) {
-	    ok = false;
-	    break;
-	}
-	// Add to circuit code list
-	for (; first <= last; first++) {
-	    if (m_count == m_chans) {
-		ok = false;
-		break;
-	    }
-	    cicCodes[m_count++] = (unsigned int)first;
-	}
-	if (!ok)
-	    break;
-    }
-    delete listSplit;
-    if (ok && m_count)
-	return cicCodes;
-    delete cicCodes;
-    m_count = 0;
-    return 0;
-}
-
 // Create circuits (all or nothing)
 // delta: number to add to each circuit code
 // cicList: Circuits to create
-bool WpData::createCircuits(unsigned int delta, const String& cicList)
+bool WpSpan::createCircuits(unsigned int delta, const String& cicList)
 {
-    if (!m_group)
-	return false;
-    // Get list. Count them
-    unsigned int* cicCodes = processCicList(cicList);
+    unsigned int* cicCodes = SignallingUtils::parseUIntArray(cicList,1,m_chans,m_count,true);
     if (!cicCodes)
 	return false;
     if (m_circuits)
@@ -1150,7 +1162,7 @@ bool WpData::createCircuits(unsigned int delta, const String& cicList)
 	    continue;
 	// Failure
 	Debug(m_group,DebugNote,
-	    "WpData('%s'). Failed to create/insert circuit %u. Rollback [%p]",
+	    "WpSpan('%s'). Failed to create/insert circuit %u. Rollback [%p]",
 	    id().safe(),cicCodes[i],this);
 	m_group->removeSpan(this,true,false);
 	delete[] m_circuits;
@@ -1165,7 +1177,7 @@ bool WpData::createCircuits(unsigned int delta, const String& cicList)
 // Read events and data from socket. Send data when succesfully read
 // Received data is splitted for each circuit
 // Sent data from each circuit is merged into one data block
-void WpData::run()
+void WpSpan::run()
 {
     if (!m_socket.open(true))
 	return;
@@ -1174,7 +1186,7 @@ void WpData::run()
 	m_buffer = new unsigned char[m_bufferLen];
     }
     XDebug(m_group,DebugInfo,
-	"WpData('%s'). Running: circuits=%u, buffer=%u, samples=%u [%p]",
+	"WpSpan('%s'). Running: circuits=%u, buffer=%u, samples=%u [%p]",
 	id().safe(),m_count,m_bufferLen,m_samples,this);
     while (true) {
 	if (Thread::check(true))
@@ -1195,13 +1207,13 @@ void WpData::run()
 	    samples = (unsigned int)r / m_count;
 	if (!samples) {
 	    Debug(m_group,DebugNote,
-		"WpData('%s'). Received data %d is not a multiple of circuit number %u [%p]",
+		"WpSpan('%s'). Received data %d is not a multiple of circuit number %u [%p]",
 		id().safe(),r,m_count,this);
 	    continue;
 	}
 	if (samples != m_samples)
 	    Debug(m_group,DebugInfo,
-		"WpData('%s'). Received %u samples. Expected %u [%p]",
+		"WpSpan('%s'). Received %u samples. Expected %u [%p]",
 		id().safe(),samples,m_samples,this);
 	unsigned char* dat = m_buffer + WP_HEADER;
 	if (m_canSend) {
@@ -1228,9 +1240,9 @@ void WpData::run()
 }
 
 // Check for received event (including in-band events)
-bool WpData::readEvent()
+bool WpSpan::readEvent()
 {
-    XDebug(m_group,DebugInfo,"WpData('%s'). Got event. Checking OOB [%p]",
+    XDebug(m_group,DebugInfo,"WpSpan('%s'). Got event. Checking OOB [%p]",
 	id().safe(),this);
     int r = m_socket.recv(m_buffer,m_bufferLen,MSG_OOB);
     if (r >= WP_HEADER)
@@ -1240,7 +1252,7 @@ bool WpData::readEvent()
 
 // Read data from socket. Check for errors or in-band events
 // Return -1 on error
-int WpData::readData()
+int WpSpan::readData()
 {
     m_buffer[WP_RD_ERROR] = 0;
     int r = m_socket.recv(m_buffer,m_bufferLen);
@@ -1248,14 +1260,14 @@ int WpData::readData()
     if (r == -1)
 	return -1;
     if (r < WP_HEADER) {
-	Debug(m_group,DebugGoOn,"WpData('%s'). Short read %u byte(s) [%p]",
+	Debug(m_group,DebugGoOn,"WpSpan('%s'). Short read %u byte(s) [%p]",
 	    id().safe(),r,this);
 	return -1;
     }
     if (m_buffer[WP_RD_ERROR]) {
 	m_readErrors++;
 	if (m_readErrors == MAX_READ_ERRORS) {
-	    Debug(m_group,DebugGoOn,"WpData('%s'). Read error %u [%p]",
+	    Debug(m_group,DebugGoOn,"WpSpan('%s'). Read error %u [%p]",
 		id().safe(),m_buffer[WP_RD_ERROR],this);
 	    m_readErrors = 0;
 	}
@@ -1267,7 +1279,7 @@ int WpData::readData()
     return r;
 }
 
-bool WpData::decodeEvent()
+bool WpSpan::decodeEvent()
 {
     return false;
 #if 0
@@ -1289,27 +1301,47 @@ bool WpData::decodeEvent()
 }
 
 /**
- * WpDataThread
+ * WpSpanThread
  */
-WpDataThread::~WpDataThread()
+WpSpanThread::~WpSpanThread()
 {
     if (m_data) {
-	DDebug(m_data->group(),DebugAll,"WpDataThread::~WpDataThread() [%p]",this);
+	DDebug(m_data->group(),DebugAll,"WpSpanThread::~WpSpanThread() [%p]",this);
 	m_data->m_thread = 0;
     }
     else
-	DDebug(DebugAll,"WpDataThread::~WpDataThread() [%p]",this);
+	DDebug(DebugAll,"WpSpanThread::~WpSpanThread() [%p]",this);
 }
 
-void WpDataThread::run()
+void WpSpanThread::run()
 {
     if (m_data) {
-	DDebug(m_data->group(),DebugAll,"WpDataThread::run() for (%p): '%s' [%p]",
-	    m_data,m_data->id().safe(),this);
+	DDebug(m_data->group(),DebugAll,"%s start running for (%p): '%s' [%p]",
+	    name(),m_data,m_data->id().safe(),this);
 	m_data->run();
     }
     else
-	DDebug(DebugAll,"WpDataThread::run(). No client object [%p]",this);
+	DDebug(DebugAll,"WpSpanThread::run(). No client object [%p]",this);
+}
+
+/**
+ * WpModule
+ */
+WpModule::WpModule()
+{
+    debugName("Wanpipe");
+    Output("Loaded module %s",debugName());
+    Configuration cfg(Engine::configFile("wpcard"));
+    cfg.load();
+
+    int level = cfg.getIntValue("general","debuglevel");
+    if (level > 0)
+	debugLevel(level);
+}
+
+WpModule::~WpModule()
+{
+    Output("Unloading module %s",debugName());
 }
 
 }; // anonymous namespace
